/*
 * This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */


/**
 * 
 */
package bit.lsi.sar.nsg;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.HierarchyBoundsListener;
import java.awt.event.HierarchyEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JComboBox;
import javax.swing.JDesktopPane;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JViewport;
import javax.swing.ListCellRenderer;
import javax.swing.SpringLayout;
import javax.swing.ToolTipManager;
import javax.swing.filechooser.FileFilter;

import au.com.bytecode.opencsv.CSVWriter;
import bit.lsi.sar.nsg.mousemenus.NSGMouseMenuManager;
import bit.lsi.sar.nsg.pathbuilder.transformers.LayoutConstants;
import bit.lsi.sar.nsg.swing.MolInfo;
import bit.lsi.sar.nsg.swing.MoleculeInfoFrame;
import bit.lsi.sar.nsg.swing.NSGFrame;
import bit.lsi.sar.nsg.swing.SwingConsole;
import edu.uci.ics.jung.graph.Graph;



/**
 * @author lounkine
 *
 */
public class JavaNSG extends JFrame{

	/**
	 * Whatever that is...
	 */
	private static final long serialVersionUID = 7402768442171683800L;
	
	
	private JDesktopPane desktop = new JDesktopPane();
	
	private JMenuBar menuBar;
	private JMenu menuFile;
	private JMenu menuHelp;
	private JPanel leftPanel;
	
	private JComboBox cbMolecules;
	private JButton btShowMolInfo = new JButton("Show Molecule Info");
	
	private JComboBox cbTarget;
	private JList curSetList;
	private DefaultListModel curSetListModel;
	
	private Target curT1;
	private Target curT2;

	
	
	private Runtime rtime = Runtime.getRuntime();
	private JScrollPane desktopscroll;
	
	private JButton bAdd = new JButton("Add to current set");
	private JButton bMakeNSG = new JButton("Show graph");
	private JButton bMakeCSV = new JButton("Write CSV");
	
	
	
	private short numSARTargetsInSet = 0;
	
	
	private NSGReader nsgReader;
	
	private JFileChooser fileChooser = new JFileChooser();
	
	private JComboBox cbCutoffs;
	
	
	
	class CutoffSelector extends JDialog { /**
		 * 
		 */
		private static final long serialVersionUID = 6655296562563239682L;

		CutoffSelector(String[] cutoffs) {
		
			super();
			
			this.setTitle("Select Cutoff");
			
			JPanel p = new JPanel(new GridLayout(3,1));
			p.setPreferredSize(new Dimension(100,100));
			
			
			p.add(new JLabel("Cutoff:"));
			cbCutoffs = new JComboBox(cutoffs);
			p.add(cbCutoffs);
			
			JButton btCutOK = new JButton("OK");
			p.add(btCutOK);
			
			this.add(p);
			
			final JDialog me = this;
			
			btCutOK.addActionListener(new ActionListener() {
				
				@Override
				public void actionPerformed(ActionEvent e) {
					me.dispose();
					
				}
			});
			
			this.pack();
			this.setModalityType(Dialog.ModalityType.APPLICATION_MODAL);
		
			
		}
	
		
		
		
	}
	
	
	private boolean allSARsets (NSGFrame caller) {
		
		numSARTargetsInSet = 1;
		
		Collection<Molecule> ms = caller.getPickedMolecules();
		if (ms.size() == 0) 
			return false;
	
		
		Target[] tsa = new Target[1];
		
		
		
		desktop.removeAll();
		desktop.repaint();
		
		for (Target t: nsgReader.getSARTargets()) {
			
			if (t.getMoleculeSet().containsAll(ms)) {
				tsa[0] = t;
				curT1 = t;
				makeNSG(tsa);
				
			}
		}
		
		for (JInternalFrame curF : desktop.getAllFrames()) {
			
			if (!curF.getClass().getSimpleName().equals("NSGFrame"))
				continue;
			
			((NSGFrame) curF).selectBasedOnMoleculeSet(ms);
			
			
		}
		
		
		numSARTargetsInSet = 0;
		return true;
		
	}
	
	
	private boolean makeCSV(Target[] ts) {
		
		JFileChooser csvChooser = new JFileChooser();
		csvChooser.addChoosableFileFilter(new CsvFileFilter());
		
		int result = csvChooser.showSaveDialog(this);
		
		if (result != JFileChooser.APPROVE_OPTION)
			return false;
		
		
		NSG nsg = getNSG(ts);
		File file = csvChooser.getSelectedFile();
		
		
		//Prepare the list to write
		List<String[]> tablelist = new ArrayList<String[]>();
		
		String[] header = {"SMILES", "ID", nsg.getCentralPropName(), "Local Score", "Sets", "Number of Neighbors", "Neighbor IDs", "Neighbor Tc Values"};
		 
		
		String[] gS = nsg.getGlobalScores();
		gS[0] = "Cont.: " + gS[0];
		gS[1] = "Disc.: " + gS[1];
		gS[2] = "SARI.: " + gS[2];
		
		tablelist.add(gS);
		tablelist.add(nsg.getTragetNames());
		tablelist.add(header);
		
		Graph<NSG.NSGVertex, NSG.NSGEdge> g = nsg.getGraph();
		
		NSG.NSGEdge curEdge;
		Collection<Target> mtargets;
		Collection<Target> nsgadd = nsg.allAddTargets();
		for (NSG.NSGVertex v1 : g.getVertices()) {
			
			String nbrids = "";
			String nbrtcs = "";
			
			mtargets = v1.getMol().getTargets();
			mtargets.retainAll(nsgadd);
			
			String sets = mtargets.toString();
			
			
			
			for (NSG.NSGVertex v2 : g.getNeighbors(v1)) {
				
				nbrids += v2.getMol().getExternalId() + " ";
				curEdge = g.findEdge(v1, v2);
				nbrtcs += String.format("%.3f", curEdge.getSim()) + " ";
										
			}
			
			String[] curLine = {v1.getMol().getSmiles(),
						v1.getMol().getExternalId(),
						String.valueOf(nsg.getCentralProperty(v1)),
						String.valueOf(nsg.getLocalScore(v1)),
						sets,
						String.valueOf(g.getNeighborCount(v1)),
						nbrids,
						nbrtcs
						
				};
			
			tablelist.add(curLine);
				
		}
		
		
		try {
			CSVWriter csvWriter = new CSVWriter(new FileWriter(file), ',', CSVWriter.DEFAULT_QUOTE_CHARACTER);
			
			csvWriter.writeAll(tablelist);
			
			csvWriter.close();
			
	} catch (IOException e) {
			
		System.err.println("Error writing csv file.");
		return false;
	}
		
		
	this.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	
	System.err.println("CSV File successfully written.");
	
	bMakeNSG.setEnabled(false);
	bMakeCSV.setEnabled(false);
	
		return true;
		
	}
	
	
	private boolean allSSRsets (NSGFrame caller) {
		
		numSARTargetsInSet = 2;
		
		Collection<Molecule> ms = caller.getPickedMolecules();
		if (ms.size() == 0) 
			return false;
		
		Target[] tsa = new Target[2];
		
		desktop.removeAll();
		desktop.repaint();
		
		for (Target t1: nsgReader.getSARTargets()) {
			
			for (Target t2: nsgReader.getSARTargets()) {
				
				if (t1.compareTo(t2) >=0)
					continue;
			
				if (t1.getMoleculeSet().containsAll(ms) && t2.getMoleculeSet().containsAll(ms)) {
					
					MoleculeSet curMs = t1.getMoleculeSet();
					curMs.retainAll(t2.getMoleculeSet());
					
					if (curMs.size() < 5)
						continue;
					
					tsa[0] = t1; tsa[1] = t2;
					curT1 = t1; curT2 = t2;
					
					makeNSG(tsa);
					
					
				}
			}
		}
		
		for (JInternalFrame curF : desktop.getAllFrames()) {
			
			if (!curF.getClass().getSimpleName().equals("NSGFrame"))
				continue;
			
			((NSGFrame) curF).selectBasedOnMoleculeSet(ms);
			
		}
		
		numSARTargetsInSet = 0;
		
		return true;
		
	}
	
	private void synchro (NSGFrame caller) {
		
		JInternalFrame[] allFrames = desktop.getAllFrames();
		
		
		Collection<Molecule> ms = caller.getPickedMolecules();
		
		for (JInternalFrame curFrame : allFrames) {
			if (curFrame.equals(caller))
				continue;
			
			if (!curFrame.getClass().equals(caller.getClass()))
				continue;
			
			((NSGFrame) curFrame).selectBasedOnMoleculeSet(ms);
		}
		
	}
	
	
	private NSG getNSG(Target[] targets) {
		
		this.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		
		int nummols = 0;
		//clear memory
		rtime.gc();
		
		SARSet sarset;
		
		if (numSARTargetsInSet == 1){		
		
			nummols = curT1.getNumMols();
			
			System.err.println("Target "+ curT1 + " has got " + nummols + " molecules");
			System.err.println("Making SARset... please hold on.");
			sarset = curT1.getSARSet();
			
			
			
		}
		
		else if (numSARTargetsInSet == 2) {
						
			sarset = SSRSet.getSSRSet(curT1, curT2);
			
			if (sarset == null || sarset.size() == 0) 
				return null;
			nummols = sarset.size();
		}
		else {
			
			System.err.println("There must be at least one Target with SAR information!");
			return null;
		}
		
		
		
		System.err.println("The Graph contains " + nummols + " Molecules.");
		if (nummols > 500)
			System.err.println("This may take long...");
		else if (nummols > 200)
			System.err.println("This might take a while...");

		
		return new NSG(sarset);
		
		
	}
	
	private NSGFrame makeNSG(Target[] targets) {
		
		NSG nsg = getNSG(targets);
		
		nsg.getSarSet().makeAllCDKMols();
		
		int nummols = nsg.getNumberMolecules();
		
		for (Target t : targets) {
				if (t.hasPotencyAnnotation())
					continue;
				
				nsg.addSet(t, t.getColor(), t.getThreshold());
				
			}
			
		
		final NSGFrame curFrame = new NSGFrame(nsg, desktop, (nummols < 100));
		curFrame.setPreferredSize(new Dimension(900,900));
		curFrame.setSize(new Dimension(900,900));
		
		curFrame.addSynchroButtonListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				// Synchronize with this frame
				synchro(curFrame);
				
			}
			

			
		});
		
		/*
		curFrame.addSARButtonListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				// TODO Auto-generated method stub
				allSARsets(curFrame);
			}
			
			
			
			
		});
		
		curFrame.addSSRButtonListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				// TODO Auto-generated method stub
				allSSRsets(curFrame);
			}
			
			
			
			
		});
		
		*/
		
		/*
		curFrame.addMoleculeDepictionMouseListener(new MouseAdapter() {

			@Override
			public void mouseClicked(MouseEvent e) {
				
				if (e.getClickCount() == 2)				
						showMoleculeInfo(curFrame.getCurrentlyDepictedMol()).toFront();
						
					
				
			
			}
			
		});
		*/
		
		
		curFrame.pack();
		
		curFrame.setVisible(true);
		
			
		desktop.add(curFrame);
		
		this.setCursor(Cursor.getDefaultCursor());
		
		bMakeNSG.setEnabled(false);
		bMakeCSV.setEnabled(false);
		
		return curFrame;
		
	}
	
	
	private void updateCombo () {
		
		//System.err.println("UpdateCombo: Number TSAR: " + numSARTargetsInSet);
		if (numSARTargetsInSet == 1){
			
			Collection<Target> cotargets = SSRSet.allSSRSetCoTargets(5, curT1, nsgReader.getAllTargets());
			
			//System.err.println("Cotargets: " + cotargets);
			
			
			ArrayList<Target> ts = new ArrayList<Target>(nsgReader.getADDTargets());
			
			for (Target t : cotargets)
				ts.add(t);
			
			Collections.sort(ts);
			
			cbTarget.removeAllItems();
			
			for (Target t: ts)
				cbTarget.addItem(t);
			
			
			
		}
		else if (numSARTargetsInSet == 0) {
			
			ArrayList<Target> ts = new ArrayList<Target>(nsgReader.getAllTargets());
			
			Collections.sort(ts);
			
			cbTarget.removeAllItems();
			
			for (Target t: ts)
				cbTarget.addItem(t);
		}
		else if (numSARTargetsInSet == 2) {
			
			ArrayList<Target> ts = new ArrayList<Target>(nsgReader.getADDTargets());
			
			Collections.sort(ts);
			
			cbTarget.removeAllItems();
			
			for (Target t: ts)
				cbTarget.addItem(t);
			
		}
		
		for (int i = 0; i < curSetListModel.getSize(); i++) {
			cbTarget.removeItem(curSetListModel.get(i));
		}
		
	}
	
	
	class CurSetListRenderer extends JLabel implements ListCellRenderer {
	    
	     // This is the only method defined by ListCellRenderer.
	     // We just reconfigure the JLabel each time we're called.

	     /**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		public Component getListCellRendererComponent(
	       JList list,
	       Object value,            // value to display
	       int index,               // cell index
	       boolean isSelected,      // is the cell selected
	       boolean cellHasFocus)    // the list and the cell have the focus
	     {
	         String s = value.toString();
	         setText(s);
	         
	   	   if (isSelected) {
	           setBackground(list.getSelectionBackground());
		       setForeground(list.getSelectionForeground());
		   }
	         else {
		       setBackground(list.getBackground());
		       setForeground(list.getForeground());
		   }
	   	   
	   	   if (value.getClass().getSimpleName().equals("Target")){
	   	   
		   	   if ( ((Target) value).hasPotencyAnnotation() )
		   		   setForeground(Color.black);
		   	   else {
		   		  
		   		   setForeground(((Target) value).getColor());
		   		   setText(s + " (" + ((Target) value).getThreshold() + ")");
		   	   }
	   	   
	   	   }
	   	   
	   	   
		   setEnabled(list.isEnabled());
		   setFont(list.getFont());
	       setOpaque(true);
	       return this;
	     }
	 }
	
	private void initLeftPanel() {
		
		
		SARSet.setNorm(nsgReader.getNormalizationConstants());
		SSRSet.setNorm(nsgReader.getNormalizationConstants());
		
		LayoutConstants.setShellThresholds(nsgReader.getRadials());
		
		
		desktop.removeAll();
		this.getContentPane().remove(leftPanel);
		leftPanel = new JPanel(new GridLayout(4,1));
		
		cbMolecules = nsgReader.getMoleculesComboBox(); 
		
		JPanel molPanel = new JPanel(new GridLayout(2,1));
		
		
		molPanel.add(cbMolecules);
		molPanel.add(btShowMolInfo);
		
		
		leftPanel.add(molPanel);
		
		JPanel comboPanel = new JPanel(new GridLayout(2,1));
		
		cbTarget = nsgReader.getTargetsComboBox();
		
		
		
		comboPanel.add(cbTarget);
		comboPanel.add(bAdd);
		
		
		curSetListModel.removeAllElements();
		curSetList = new JList(curSetListModel) {

			/**
			 * 
			 */
			private static final long serialVersionUID = 1L;
			
			public String getToolTipText(MouseEvent evt) {
	            // Get item index
	            int index = locationToIndex(evt.getPoint());
	            
	            if (index >= 0)
	            {
		            // Get item
		            Object item = getModel().getElementAt(index);
		    
		            // Return the tool tip text
		            return item.toString();
	            }
	            else return "";
	        }			
			
			
		};
		curSetList.setPrototypeCellValue(cbTarget.getPrototypeDisplayValue());
		
		curSetList.setCellRenderer(new CurSetListRenderer());
		
		
		
		curSetList.addKeyListener(new KeyListener() {

			@Override
			public void keyPressed(KeyEvent e) {
				// TODO Auto-generated method stub
				if (e.getKeyCode() == KeyEvent.VK_DELETE) {
					
					int i = curSetList.getSelectedIndex();
					
					Object curT =  curSetList.getModel().getElementAt(i);
					
					if (curT.equals(curT1) || curT.equals(curT2)) {
						
						if (curT.equals(curT1)) {
						curT1 = curT2;
						curT2 = null;
						}
						
						numSARTargetsInSet--;
						
					}
					
					
					curSetListModel.remove(i);
					
					updateCombo();
					
				}
			}

			@Override
			public void keyReleased(KeyEvent arg0) {
				
				
			}

			@Override
			public void keyTyped(KeyEvent e) {
				// Remove set from list
				
				
			}
			
		});
		
		
		leftPanel.add(comboPanel);
		
		JScrollPane spList = new JScrollPane(curSetList);
		
		
		
		leftPanel.add(spList);
		
		
		JPanel pLowerP = new JPanel(new GridLayout(2,1));
		
		bMakeNSG.setEnabled(false);
		bMakeCSV.setEnabled(false);
		
		pLowerP.add(bMakeNSG);
		pLowerP.add(bMakeCSV);
		
		
		
		
		leftPanel.add(pLowerP);
		
		leftPanel.setPreferredSize(new Dimension(150, 600));
		leftPanel.setSize(leftPanel.getPreferredSize());
		
		this.getContentPane().add(leftPanel);
		this.pack();
	}
	
	
	
	private String getExtension(File f) {
	       String ext = null;
	       String s = f.getName();
	       int i = s.lastIndexOf('.');

	       if (i > 0 &&  i < s.length() - 1) {
	           ext = s.substring(i+1).toLowerCase();
	       }
	       return ext;
	   }
	
	class ZipFileFilter extends FileFilter {

		@Override
		public boolean accept(File f) {
			// TODO Auto-generated method stub
			
			if (f.isDirectory())
				return true;
			
			String extension = getExtension(f);
			
			if (extension != null) {
				
				if (extension.equals("zip"))
					return true;
				
			}
						
			return false;
		}

		@Override
		public String getDescription() {
			// TODO Auto-generated method stub
			return "*.zip: Zip files";
		}
		
		
		
		
	}
	
	
	class CsvFileFilter extends FileFilter {

		@Override
		public boolean accept(File f) {
			// TODO Auto-generated method stub
			
			if (f.isDirectory())
				return true;
			
			String extension = getExtension(f);
			
			if (extension != null) {
				
				if (extension.equals("csv"))
					return true;
				
			}
						
			return false;
		}

		@Override
		public String getDescription() {
			// TODO Auto-generated method stub
			return "*.csv: Comma Separated Files";
		}
		
		
		
		
	}
	
	
	private boolean openFile () {
		
		
		leftPanel.setEnabled(false);
		fileChooser.addChoosableFileFilter(new ZipFileFilter());
		
		
		
		int returnVal = fileChooser.showOpenDialog(this);
		
		if (returnVal != JFileChooser.APPROVE_OPTION) 
			return false;
		
	File file = fileChooser.getSelectedFile(); 
		
		
		try {
			
			CutoffSelector cs = new CutoffSelector(NSGReader.getAvailableCutoffs(file.getAbsolutePath())); 
			cs.setModalExclusionType(Dialog.ModalExclusionType.APPLICATION_EXCLUDE);
			cs.setVisible(true);
			
			nsgReader = new NSGReader(file.getAbsolutePath(), cbCutoffs.getSelectedItem().toString());
			//DEBUG:
			//nsgReader = new NSGReader("l:/Work/temp/nsgdebug/nsgdebug.zip");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}
		leftPanel.setEnabled(true);
		return true;
		
	}
	
	private void showAboutDialogue() {
		
		JOptionPane.showMessageDialog(this, "SARANEA v1.0, Copyright (C) 2009 Eugen Lounkine" +
				"\nSARANEA comes with ABSOLUTELY NO WARRANTY;\nfor details, refer to the GNU Public License provided with the software or under\nhttp://www.gnu.org/licenses/gpl-2.0.txt.", "About", JOptionPane.INFORMATION_MESSAGE);
		
	}
	
	private void initMenu() {
		
		menuBar = new JMenuBar();
		
		menuFile = new JMenu("File");
		menuFile.setMnemonic(KeyEvent.VK_F);
		menuFile.getAccessibleContext().setAccessibleDescription("File Menu");
		
		
		JMenuItem menuFileOpen = new JMenuItem();
		menuFileOpen.setText("Open");
		menuFile.add(menuFileOpen);
		
		menuHelp = new JMenu("Help");
		JMenuItem menuHelpAbout = new JMenuItem();
		menuHelpAbout.setText("About");
		menuHelp.add(menuHelpAbout);
		
		
		menuFileOpen.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				
				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				
				if (openFile())
					initLeftPanel();
				
				setCursor(Cursor.getDefaultCursor());
				
			}
			
		});
		
		menuHelpAbout.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent arg0) {
				showAboutDialogue();
				
			}
		});
		
			
			
			
	
		
		
		//progressBar.setStringPainted(true);
		
		menuBar.add(menuFile);
		menuBar.add(menuHelp);
		//menuBar.add(progressBar);
		
		
		
		this.setJMenuBar(menuBar);
		
		
	}
	
	
	private MoleculeInfoFrame showMoleculeInfo(final Molecule m) {
		
		final MoleculeInfoFrame mif = new MoleculeInfoFrame(m);
		desktop.add(mif);
		
		mif.addTableMouseListener(new MouseAdapter() {
			
			public void mouseClicked(MouseEvent e2) {
				
				if (e2.getClickCount() == 2) {
					
					MolInfo mi = mif.getMolInfo();
					Target[] ts = mi.getTargets();
					
					if (ts[0].hasPotencyAnnotation()) {
					
						Target oldt1 = curT1;
						Target oldt2 = curT2;
						
						curT1 = ts[0];
						if (ts.length == 2) 
							curT2 = ts[1];
						else
							curT2 = null;
						
						short oldTSAR = numSARTargetsInSet;
						numSARTargetsInSet = (short) ts.length;
						
						Collection<Molecule> ms = new ArrayList<Molecule>(1);
						ms.add(m);
						
						makeNSG(ts).selectBasedOnMoleculeSet(ms);
						
						
						numSARTargetsInSet = oldTSAR;
						curT1 = oldt1;
						curT2 = oldt2;
					}
					else
						System.err.println("Cannot make NSG from Additional Set.");
				}
					
			}
			
			
		});
		
		mif.toFront();
		desktop.repaint();
		
		return mif;
	}
	
	private void addListeners() {
		
		//Add listeners to buttons
		final JFrame me = this;
		
		
		this.getContentPane().addHierarchyBoundsListener(new HierarchyBoundsListener() {
			
			@Override
			public void ancestorResized(HierarchyEvent e) {
				
				desktopscroll.setPreferredSize(new Dimension (me.getContentPane().getWidth() - 50, me.getContentPane().getHeight() - 10));
				desktopscroll.revalidate();
			}
			
			@Override
			public void ancestorMoved(HierarchyEvent e) {
				// TODO Auto-generated method stub
				
			}
		});
		
		btShowMolInfo.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				
				Molecule m = (Molecule) cbMolecules.getSelectedItem();
				
				showMoleculeInfo(m).toFront();
				
				
			}
			
		});
		
		
		bAdd.addActionListener( new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
							
				
				bMakeNSG.setEnabled(true);
				bMakeCSV.setEnabled(true);
				
				Target t = (Target) cbTarget.getSelectedItem();
				
				if (t.hasPotencyAnnotation()) {
					if (numSARTargetsInSet < 2)	{
						numSARTargetsInSet++;
						
						curSetListModel.addElement(t);
						
						if (numSARTargetsInSet == 1) {
							curT1 = t;
							//curT1idx = curSetListModel.size() - 1;
						}
						else {
												
							curT2 = t;
							//curT2idx = curSetListModel.size() - 1;
						}
						
						updateCombo();
											
							
					}
					else {
						System.err.println("Maximally two Targets with SAR Information allowed!");
					}
					
				}
				else {
					
					Color col = JColorChooser.showDialog(me, "Select Color for additional Set " + t.toString(), Color.red);
					t.setColor(col);
					
					double threshold = nsgReader.getCutoff();
					
					
					String response = JOptionPane.showInputDialog(me,
						  "Which Threshold?",
						  "Threshold selection",
						  JOptionPane.QUESTION_MESSAGE);
						
					try {
						threshold = Double.parseDouble(response);
					}
					catch (NumberFormatException e) {
						
						threshold = nsgReader.getCutoff();
						
					};
					
					if (threshold < 0 || threshold > 1) {
						System.err.println("Using default");
						threshold = nsgReader.getCutoff();
						
					}
						
									
					
					
					t.setThreshold(threshold);
						
					curSetListModel.addElement(t);
					cbTarget.removeItem(t);
					
				}
					
			}
			
			
			
			
		});
		
		
		bMakeNSG.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				
				Target t;
				Target[] ts = new Target[curSetListModel.size()];
				for (int i = 0; i < curSetListModel.size(); i++) {
						
					t = (Target) curSetListModel.getElementAt(i);
					
					ts[i] = t;
					
				}
				
				makeNSG(ts).toFront();
				
				curSetListModel.clear();
				numSARTargetsInSet = 0;
				updateCombo();
				
				
			}
			
		});
		
		
		bMakeCSV.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				
				Target t;
				Target[] ts = new Target[curSetListModel.size()];
				for (int i = 0; i < curSetListModel.size(); i++) {
						
					t = (Target) curSetListModel.getElementAt(i);
					
					ts[i] = t;
					
				}
				
				if (makeCSV(ts))
				{
				
					curSetListModel.clear();
					numSARTargetsInSet = 0;
					updateCombo();
				}
				
			}
			
		});
		
		
		NSGMouseMenuManager.sharedInstance().menuItemMI.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				showMoleculeInfo(NSGMouseMenuManager.sharedInstance().menuItemMI.getVertex().getMol());
			}
		});
		
		
	}
	
	
	private void initListModel() {
		
		curSetListModel = new DefaultListModel();
			
			/*new AbstractListModel() {

			
			private static final long serialVersionUID = -7925497111539411669L;

			@Override
			public Object getElementAt(int i) {
				// TODO Auto-generated method stub
				return curSet.get(i);
			}

			@Override
			public int getSize() {
				// TODO Auto-generated method stub
				return curSet.size();
			}

			
			
		};*/
		
	}
	
	
	
	
	
	public JavaNSG () {
		
		super();
		
		Container content = this.getContentPane();
		
		content.setLayout(new BoxLayout(content, BoxLayout.LINE_AXIS));
		
				
		
		desktop.setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);
		
		//JPanel overAllPanel = new JPanel();
		//overAllPanel.setLayout(new BorderLayout());
		
		desktopscroll = new JScrollPane();
		JViewport viewport = new JViewport();
		
		//overAllPanel.add(desktop, BorderLayout.CENTER);
		
		viewport.setView(desktop);
		
		desktopscroll.setViewport(viewport);
		
		desktopscroll.setPreferredSize(new Dimension(950,950));
		
		
		
		desktop.setPreferredSize(new Dimension(2000,1800));
		
		
		//overAllPanel.add(desktopscroll);
		content.add(desktopscroll);
		
		//content.add(overAllPanel);
		leftPanel = new JPanel(new GridLayout(3,1));
		content.add(leftPanel);
		
		
		
		initMenu();
		initListModel();
		addListeners();

		ToolTipManager.sharedInstance().setInitialDelay(10);
		ToolTipManager.sharedInstance().setDismissDelay(Integer.MAX_VALUE);
		
		this.setTitle("SARANEA");
		
		
		
		this.pack();
		
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
	}
	

	/**
	 * @param args
	 * 
	*/
	public static void main(String[] args) {
	
		SwingConsole.run(new JavaNSG(), 600,600);
		
						
	}

}
