/*
 * This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package bit.lsi.sar.nsg;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;

import java.awt.Container;


import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.zip.*;

import javax.swing.JComboBox;
import javax.swing.JDesktopPane;

import javax.swing.JFrame;
import javax.swing.JList;

import javax.swing.JPanel;
import javax.swing.plaf.basic.BasicComboBoxRenderer;

import org.apache.commons.collections15.Transformer;

import cern.colt.bitvector.BitVector;


import bit.lsi.sar.nsg.swing.NSGFrame;
import bit.lsi.sar.nsg.swing.SwingConsole;


import au.com.bytecode.opencsv.CSVReader;

public class NSGReader {
	
	
	private ZipFile zfin; 
	
	private HashMap<String,Double> normsettings = new HashMap<String,Double>();
	private double[] radials = {0.90,0.85,0.80,0.75,0.70,0}; 
	
	private MoleculeSet basisset = new MoleculeSet();
	private Set<Target> targets = new HashSet<Target>();
	
	

	private HashMap<String, Integer> fpFeatureMap = new HashMap<String, Integer>();
	
	
	private String settingsfn = "065";
	
	
	private Reader getZIPReader(String zipentry) throws IOException {
		
		ZipEntry ze = zfin.getEntry(zipentry);
		return new InputStreamReader(zfin.getInputStream(ze));
		
	}
	
	private Reader getZIPReader(ZipEntry ze) throws IOException {
		
		return new InputStreamReader(zfin.getInputStream(ze));
		
	}
	
	
	private void collectFPfeatures () {
		
		Set<String> fpFeatures = new HashSet<String>();
		Enumeration<?> zipEntries = zfin.entries();
		
		String curFP[];
		
		//Loop through all files in that folder
		while (zipEntries.hasMoreElements()) {
			
			ZipEntry curZipEntry = (ZipEntry) zipEntries.nextElement();
			String curName = curZipEntry.getName();
			
			//check if we are in the SARSets directory with our entry
			if (!(curName.contains("Sets") && curName.length() > 8))
				continue;
			
			
			
			String targetName = curName.substring(8);
			targetName = targetName.substring(0,targetName.length()-4);	//assuming they end with .csv, which they should!
			
			//Open the file
			NSGTableReader tr;
			try {
				tr = new NSGTableReader(getZIPReader(curZipEntry));
			
			
			//loop through the entries in the file
			HashMap<String,String> curEntry;
					
			
				while ((curEntry = tr.nextEntry()) != null) {
					
					curFP = curEntry.get("FP").split(" ");
					
					for (String curKey : curFP) {
						fpFeatures.add(curKey);
					}
					
									
					
				}
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} //while ((curEntry = tr.nextEntry()) != null)
			
		}//Loop through all files in that folder
		
		int i = 0;
		for (String curKey: fpFeatures) {
			fpFeatureMap.put(curKey, i++);
		}
		
	}
	
	private BitVector featureStringToBitVector (String fpString){
		
		
		
		
		BitVector bt = new BitVector(fpFeatureMap.size());
		String[] keys = fpString.split(" ");
		
		for (String curKey : keys)
			bt.set(fpFeatureMap.get(curKey));
		
		//System.err.println(bt.toString());
		
		
		return bt;
		
	}
	
	
	private void loadDefaults() {
		
		normsettings.put("globalSARIContMean",0.6139808);
		normsettings.put("globalSARIContSd",0.05266367);
		normsettings.put("globalSARIDiscMean",1.070078);
		normsettings.put("globalSARIDiscSd",0.5064047);
		normsettings.put("globalSSRIContMean",0.6114327);
		normsettings.put("globalSSRIContSd",0.05559916);
		normsettings.put("globalSSRIDiscMean",1.052148);
		normsettings.put("globalSSRIDiscSd",0.55098);
		normsettings.put("localSARImean",0.6081146);
		normsettings.put("localSARIsd",0.3583711);
		normsettings.put("localSSRImean",0.6065776);
		normsettings.put("localSSRIsd",0.4159975);
		normsettings.put("CUTOFF",0.65);
	}
	
	private void readSettings() {
		
		loadDefaults();
		
		System.err.println("Reading settings...");
		
		Transformer<String, double[]> doublesplitter = new Transformer<String, double[]>() {

			@Override
			public double[] transform(String arg0) {
				String[] values = arg0.split(" ");
				double[] vals = new double[values.length];
				for (int i = 0; i < vals.length; i++) {
					vals[i] = Double.parseDouble(values[i]);
				}
				
				return vals;
			}
			
		};
		
		//Read normalization scores
		try {
			
			CSVReader csvr = new CSVReader(getZIPReader("Settings/" + settingsfn + ".csv"));
			String[] nextLine;
			
			while ((nextLine = csvr.readNext()) != null) {
		       if (nextLine[0].equals("RADIALS")) {
		    	   
		    	   radials = doublesplitter.transform(nextLine[1]);
		    	   
		       }
		       else
		    	   normsettings.put(nextLine[0], Double.parseDouble(nextLine[1]));
			}
					
		} catch (Exception e) {
			System.err.println("Could not open settings, using defaults:");
			System.err.println(this.normsettings);
			e.printStackTrace();
		}
	}
	
	
	
	
	private Molecule addMol(Molecule mol) {
		
		//System.err.println("Trying to add " + mol + " with SMILES " + mol.getSmiles() + "...");	
		
		if (!basisset.add(mol)) {
			
				//Molecule was already in the set
				//Merge the potencies of this molecule with the potencies of the molecule in the set
				
			//System.err.println("The Baisset already contains " + mol + " !");	
			
		
			
			Iterator<Molecule> it = basisset.iterator();
				while(it.hasNext()) {
					Molecule foundMol = it.next();
					
					if (foundMol.equals(mol)){
						//Mol found!
						//Merge the potencies of the foundMol with the ones given here
						
						HashSet<Target> newTargets = mol.getTargets();
						
						for (Target curT : newTargets) {
							
							//If the molecule already has a potency for this target, leave it alone
							if (!foundMol.getTargets().contains(curT)) {
								foundMol.addPotency(curT, mol.getPotency(curT));
							}
						}
						
						
						//Merge Descriptors
						//If a descriptor is present already, it's value is not touched
						
						HashSet<String> newD= mol.getDescriptorNames();
						
						for (String dName : newD) {
							
							if (foundMol.getDescriptorValue(dName) == null) {
								//Descriptor unknown
								foundMol.addDescriptor(dName, mol.getDescriptorValue(dName));
							}
						}
						
						
						
						return foundMol; //while(it.hasNext())
					}
				}
		}
		
		return mol;
	}
	
	private void readSARSets() throws IOException {
		
			
			
			System.err.println("Reading SAR Sets...");
		
			Enumeration<?> zipEntries = zfin.entries();
			
			//Loop through all files in that folder
			while (zipEntries.hasMoreElements()) {
				
				
				
				
				ZipEntry curZipEntry = (ZipEntry) zipEntries.nextElement();
				String curName = curZipEntry.getName();
				
				//check if we are in the SARSets directory with our entry
				if (!(curName.contains("SARSets") && curName.length() > 8))
					continue;
				
				
				
				String targetName = curName.substring(8);
				targetName = targetName.substring(0,targetName.length()-4);	//assuming they end with .csv, which they should!
				
				//System.err.println("Current Target Name = " + targetName);
				
				//Initialize target with target name
				Target curTarget = new Target(targetName);
				targets.add(curTarget);
				
				System.err.print("Reading Target " + curTarget + "... ");
				
				
				//Open the file
				NSGTableReader tr = new NSGTableReader(getZIPReader(curZipEntry));
				
				//loop through the entries in the file
				HashMap<String,String> curEntry;
				
				while ((curEntry = tr.nextEntry()) != null) {
					
					//Make new molecule from entry
					Molecule newMol = new Molecule(
									curEntry.get("SMILES"),
									curEntry.get("ID"),
									curEntry.get("NAME"),
									featureStringToBitVector(curEntry.get("FP"))
							);
					
									
					newMol.addPotency(
							curTarget,
							new Potency ( Double.parseDouble( curEntry.get("POT") ) )
						);
					
					//Remove all reserved words
					curEntry.remove("SMILES");
					curEntry.remove("ID");
					curEntry.remove("NAME");
					curEntry.remove("FP");
					curEntry.remove("POT");
					
					//Add remaining fields as descriptors
					for (String propName : curEntry.keySet())
						newMol.addDescriptor(propName, curEntry.get(propName));
					
					
					newMol = addMol(newMol);
					
					for (Target t : newMol.getTargets())
						t.addMolecule(newMol);
					
					
				} //while ((curEntry = tr.nextEntry()) != null)
				
				System.err.println( "done.");	
				
			}//Loop through all files in that folder
			
	}
	
	
	
	private void readADDSets() throws IOException {
		
		Enumeration<?> zipEntries = zfin.entries();
		
		//Loop through all files in that folder
		while (zipEntries.hasMoreElements()) {
			
			ZipEntry curZipEntry = (ZipEntry) zipEntries.nextElement();
			String curName = curZipEntry.getName();
			
			//check if we are in the SARSets directory with our entry
			if (!(curName.contains("ADDSets") && curName.length() > 8))
				continue;
			
			
			
			String targetName = curName.substring(8);
			targetName = targetName.substring(0,targetName.length()-4);	//assuming they end with .csv, which they should!
			
			//System.err.println("Current Additional Target Name = " + targetName);
			
			//Initialize mock target with target name
			Target curTarget = new Target(targetName, false);
			targets.add(curTarget);
			
			System.err.print("Reading Additional Target " + curTarget + "... ");
			
			//Open the file
			NSGTableReader tr = new NSGTableReader(getZIPReader(curZipEntry));
			
			//loop through the entries in the file
			HashMap<String,String> curEntry;
			
			while ((curEntry = tr.nextEntry()) != null) {
				
				//System.err.println(curEntry);
				
				//Make new molecule from entry
				Molecule newMol = new Molecule(
								curEntry.get("SMILES"),
								curEntry.get("ID"),
								curEntry.get("NAME"),
								featureStringToBitVector(curEntry.get("FP"))
						);
				
				
				//System.err.println("Molecule created");
				
				newMol.addPotency(
						curTarget,
						new Potency(1d)
					);
				
				//System.err.println("Potency added");
				
				//Remove all reserved words
				curEntry.remove("SMILES");
				curEntry.remove("ID");
				curEntry.remove("NAME");
				curEntry.remove("FP");
				curEntry.remove("POT");
				
				//Add remaining fields as descriptors
				for (String propName : curEntry.keySet())
					newMol.addDescriptor(propName, curEntry.get(propName));
				
				//System.err.println("Calling Addmol...");
				addMol(newMol);
				for (Target t : newMol.getTargets())
					t.addMolecule(newMol);
			} //while ((curEntry = tr.nextEntry()) != null)
			
			System.err.println( "done.");	
			
		}//Loop through all files in that folder
		
}
	
	private void readFiles() throws IOException {
		ZipEntry ze;
		
		
		//Read Settings
		//ze = zfin.getEntry("Settings");
		readSettings();
				
		
		//Read SAR Sets
		
		ze = zfin.getEntry("SARSets");
		
		if (ze != null) {
			
			readSARSets();
			
		}
		else {
			System.err.println("SARSets could not be found!");
			throw new IOException();
		}
		
		
		
		//Read ADD Sets
		
		ze = zfin.getEntry("ADDSets");
		
		if (ze != null) {
			
			readADDSets();
			
		}
		else {
			System.err.println("No additional Sets defined.");
		}
		
		
	}
	
	
	
	public static String[] getAvailableCutoffs(String zipFile) throws IOException {
		
		ArrayList<String> als = new ArrayList<String>(7);
		
		ZipFile zin = new ZipFile(zipFile);
		
		Enumeration<?> zipEntries = zin.entries();
		
		while (zipEntries.hasMoreElements()) {
			
			ZipEntry curZipEntry = (ZipEntry) zipEntries.nextElement();
			String curName = curZipEntry.getName();
			
			//check if we are in the SARSets directory with our entry
			if (!(curName.contains("Settings") && curName.length() > 9))
				continue;
			
			String settingName = curName.substring(9);
			settingName = settingName.substring(0,settingName.length()-4);	//assuming they end with .csv, which they should!
			als.add(settingName);
			
			
		}
		
		zin.close();
		String[] cutoffs = new String[als.size()];
		cutoffs = als.toArray(cutoffs);
			
		return cutoffs;
		
	}
	
		
	public NSGReader (String zipFile) throws IOException {
		
		this(zipFile, "065");
		
				
	}
	
	
	public NSGReader (String zipFile, String settingsfn) throws IOException {
		
		
		this.settingsfn = settingsfn;
		zfin = new ZipFile(zipFile);
			
		System.err.print("Preprocessing fingerprints. This may take a while...");
		collectFPfeatures();
		System.err.println("Done.");
		readFiles();
		
		zfin.close();
		
		fpFeatureMap = null;	//not longer needed since everything is read
		
		System.err.println("Settings, Molecules, and Targets read.");
		
				
	}
	
	
	public HashMap<String,Double> getNormalizationConstants() {
		return new HashMap<String,Double>(normsettings);
	}
	
	public double[] getRadials() {
		return radials;
	}
	
	public HashSet<Target> getAllTargets() {
		return new HashSet<Target>(targets);
	}
	
	public HashSet<Target> getSARTargets() {
		
		HashSet<Target> ts = new HashSet<Target>();
		
		for (Target t : targets) {
			if (t.hasPotencyAnnotation())
				ts.add(t);
		}
		
		return ts;
	}
		
	public HashSet<Target> getADDTargets() {
			
		HashSet<Target> ts = new HashSet<Target>();
			
		for (Target t : targets) {
			if (!t.hasPotencyAnnotation())
				ts.add(t);
		}
		return ts;
	}
	
	
	public MoleculeSet getBasisSet() {
		
		return basisset; //Caution! the reference to the private basis set is returned here!
		
	}
	
	public double getCutoff () {
		return normsettings.get("CUTOFF");
	}
	
	
	
	class NSGRComboBoxRenderer extends BasicComboBoxRenderer {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		public Component getListCellRendererComponent( JList list,
		Object value, int index, boolean isSelected, boolean cellHasFocus) {
		if (isSelected) {
			setBackground(list.getSelectionBackground());
			setForeground(list.getSelectionForeground());
		if (-1 < index) {
		list.setToolTipText(list.getModel().getElementAt(index).toString());
		}
		} else {
			setBackground(list.getBackground());
			setForeground(list.getForeground());
		}
		
		if (value != null && value.getClass().getSimpleName().equals("Target")) {
			if (!((Target) value).hasPotencyAnnotation())
				setForeground(Color.red);
		}
		
		setFont(list.getFont());
		setText((value == null) ? "" : value.toString());
		return this;
		}
	}
	
	public JComboBox getTargetsComboBox() {
		
		ArrayList<Target> ts = new ArrayList<Target>(targets.size());
		
		for (Target t: targets) {
			
			//if (t.hasPotencyAnnotation())
				ts.add(t);
		}
		
		Collections.sort(ts);
		
		
		
		Target[] tsa = new Target[ts.size()];		
		tsa = ts.toArray(tsa);
		
		//System.err.println(tsa);
		final JComboBox tcb = new JComboBox(tsa);
		tcb.setPrototypeDisplayValue("XXXXXXXXXX");
		
		tcb.setRenderer(new NSGRComboBoxRenderer());
		
		
		
		return tcb;
		
		
	}
	
	
	public JComboBox getMoleculesComboBox() {
		
		ArrayList<Molecule> ms = new ArrayList<Molecule>(basisset.size());
		
		for (Molecule m: basisset) {
			
			ms.add(m);
			
		}
		
		Comparator<Molecule> molNameComparator = new Comparator<Molecule>() {

			@Override
			public int compare(Molecule m1, Molecule m2) {
				return m1.toString().compareTo(m2.toString());
			}
			
		};
		
		Collections.sort(ms, molNameComparator);
		
		Molecule[] msa = new Molecule[ms.size()];
		msa = ms.toArray(msa);
		JComboBox mcb = new JComboBox(msa);
		mcb.setPrototypeDisplayValue("XXXXXXXXXX");
		mcb.setRenderer(new NSGRComboBoxRenderer());
		
		
		return mcb;
		
	}
	
	
	public static void main (String[] args) {
		
		NSGReader nsgr;
		try {
			
			nsgr = new NSGReader("L:/Work/temp/nsgdebug/nsgdebug.zip");
			
			SARSet.setNorm(nsgr.getNormalizationConstants());
			SSRSet.setNorm(nsgr.getNormalizationConstants());
						
			
			
			//System.out.println(nsgr.getNormalizationConstants());
			//System.out.println(nsgr.getAllTargets());
			
			
			Iterator<Target> itar = nsgr.getSARTargets().iterator();
			Iterator<Target> iadd = nsgr.getADDTargets().iterator();
			
			Target tox = iadd.next();
			
			//System.err.println(tox + " Tox Set: " + tox.getMoleculeSet());
			
			JFrame frame = new JFrame();
		    JDesktopPane desktop = new JDesktopPane();
		    Container content = frame.getContentPane();
		    JPanel panel = new JPanel(new BorderLayout());
		    panel.add(desktop);
		    content.add(panel);
		    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			
		    while (itar.hasNext()) {
				
		    	Target curTar = itar.next();
				
				
				SARSet bla = curTar.getSARSet();
				
				if (bla != null && bla.size() > 0) {
					
					System.err.println("Size of set = " + bla.size());
					
					NSG nsg = new NSG(bla);
					
					//nsg.addSet(tox, Color.red);
					
					NSGFrame vvFrame = new NSGFrame(nsg, desktop, true);
		        
					vvFrame.pack();
					vvFrame.setVisible(true); //necessary as of 1.3
					desktop.add(vvFrame);
					try {
						vvFrame.setSelected(true);
					} catch (java.beans.PropertyVetoException e) {}
				}
		
			}
		    
		    frame.pack();
		    frame.setVisible(true);
		    
		    SwingConsole.run(frame, 800, 800); 
		    
				
		}
		catch (IOException e) {
			
			e.printStackTrace();
			
		}
		
		
	}
	

}
