/*
 * This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
 * 
 */
package bit.lsi.sar.nsg;

import java.awt.Color;

import java.util.Collection;



/**
 * @author lounkine
 *
 */
public class Target implements Comparable<Target>{
	
	/**
	 * 
	 */
	//private static final long serialVersionUID = 6970760225419095878L;
	private static int targetCount = 0;
	private int internalId = 0;
	
	private Color color;
	private double threshold;
	
	private String name;
	
	private boolean withPotency = true;
	
	private MoleculeSet molecules = new MoleculeSet();
	
	
	public Target (String name) {
		
		
		this.name = name;
		
		internalId  = ++targetCount;
		
	}
	
	
	
	
	
	public Target (String name, boolean withPotency) {
		
		this.name = name;
		this.withPotency = withPotency;
		
		internalId  = ++targetCount;
		
	}
	
	
	public void finalize() {
		targetCount--;
	}
	
	public String toString() {
		
		/*
		if (withPotency) 
			return new String(name);
		else
			return new String("(add) "+ name);
		*/
		
		return name;
	}
	
	public int getId () {
		return internalId;
	}
	
	public boolean hasPotencyAnnotation() {
		return withPotency;
	}
	
	public void addMolecule(Molecule mol) {
		molecules.add(mol);
	}
	
	public boolean removeMolecule(Molecule mol) {
		return molecules.remove(mol);
	}
	
	public MoleculeSet getMoleculeSet() {
		return new MoleculeSet(molecules);
	}
	
	public int getNumMols() {
		return molecules.size();
	}
	
	public SARSet getSARSet() {
		
		if (withPotency)
			return new SARSet(molecules, this);
		else {
			System.err.println("Cannot create a SARSet from mock target!");
			return null;
		}
			
	}


	@Override
	public int compareTo(Target o) {
	
		return (this.toString().compareTo(o.toString()));
	}





	public void setColor(Color color) {
		this.color = color;
	}





	public Color getColor() {
		return color;
	}





	public void setThreshold(double threshold) {
		this.threshold = threshold;
	}





	public double getThreshold() {
		return threshold;
	}
	
	public Collection<Molecule> sharedMols (Target t2) {
		
		
		
		Collection<Molecule>t2ms = t2.getMoleculeSet();
		
		t2ms.retainAll(molecules);
		
		
		return t2ms;
		
	}
	
	

}
