/*
 * This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package bit.lsi.sar.nsg.mousemenus;

import bit.lsi.sar.nsg.swing.NSGFrame;

public class NSGMouseMenuManager {

	final static NSGMouseMenuManager nsgMMM = new NSGMouseMenuManager(); 
	
	public static NSGMouseMenuManager sharedInstance() {
		return nsgMMM;
	}
	
	public static enum State {NSG, SARTREE, PATHBUILDER}
	
	
	//--------------------------------------------------------------------------
	
	private NSGPopupGraphMousePlugin mp = new NSGPopupGraphMousePlugin();
	
	private NSGMouseMenus.VertexMenu vertexMenu = new NSGMouseMenus.VertexMenu();
	
	//Menu Items that are everywhere
	public NSGMouseMenus.VertexMenuItem menuItemMI = new NSGMouseMenus.VertexMenuItem("Molecule Info");
	public NSGMouseMenus.VertexMenuItem menuItemSelectNeighbors = new NSGMouseMenus.VertexMenuItem("Select Neighbors");
	
	//Menu Items for the NSG
	public NSGMouseMenus.VertexMenuItem menuItemSarTree = new NSGMouseMenus.VertexMenuItem("Show SAR Tree");


	//Menu Items for the Path Builder
	public NSGMouseMenus.VertexMenuItem menuItemRadial = new NSGMouseMenus.VertexMenuItem("Show Neighborhood");
	
	
	//Menu Items for the SAR Tree Viewer
	public NSGMouseMenus.VertexMenuItem menuItemShowPathInPB = new NSGMouseMenus.VertexMenuItem("Show Path in Path Builder");
	
	
	
	private State curState = State.NSG;
	private NSGFrame activeNSGFrame = null;
	
	
	private void initMenu() {
		
		vertexMenu.removeAll();
		
		vertexMenu.add(menuItemMI);
		vertexMenu.add(menuItemSelectNeighbors);
		vertexMenu.addSeparator();
		
		
		switch(curState) {
		
		case NSG:
			
			vertexMenu.add(menuItemSarTree);
			break;
			
		case PATHBUILDER:
			vertexMenu.add(menuItemRadial);
			break;
			
		case SARTREE:
			vertexMenu.add(menuItemShowPathInPB);
			break;
		
		}
		
		
		
		
		
	}
	
	
	public void setState(State s) {
		
		curState = s;
		initMenu();
		
	}
	
	public void setActiveNSGFrame(NSGFrame n) {
		this.activeNSGFrame = n;
	}
	
	public NSGFrame getActiveNSGFrame() {
		return activeNSGFrame;
	}
	
	
	public NSGMouseMenuManager() {
		
		mp.setVertexPopup(vertexMenu);
		initMenu();
		
	}
		
	public NSGPopupGraphMousePlugin getMousePlugin() {
		return mp;
	}
	
	
	
	
	
}
