/*

 * This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package bit.lsi.sar.nsg.pathbuilder;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashSet;
import java.util.List;

import bit.lsi.sar.nsg.NSG;
import bit.lsi.sar.nsg.NSG.NSGVertex;
import bit.lsi.sar.nsg.pathbuilder.transformers.LayoutConstants;
import edu.uci.ics.jung.graph.DirectedSparseGraph;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.visualization.Layer;
import edu.uci.ics.jung.visualization.VisualizationServer;
import edu.uci.ics.jung.visualization.VisualizationViewer;

public class PathBuilder {

	
	
	class Axes implements VisualizationServer.Paintable {

		
		private Line2D getYAxis () {
			
			
			
			Point2D start;
			Point2D end;
			
			double x = 0;
			double y = -5 + LayoutConstants.yOffset + LayoutConstants.nodeYSpaceFactor * (getMaxCentralValue() - getMinCentralValue());
			
			start = new Point2D.Double(x,  0);
			end = new Point2D.Double(x, y);
				
			Line2D yAxis = new Line2D.Double(start, end);
			
			return yAxis;
			
			
		}
		
		private Line2D[] getYTicks() {
			
			Line2D[] ticks = new Line2D[path.size()];
			
			Point2D start;
			Point2D end;
			
			double y = 0;
			
			for (int i = 0; i < ticks.length ; i++ ) {
				
				y =  LayoutConstants.yOffset + LayoutConstants.nodeYSpaceFactor * ( nsg.getCentralProperty(path.get(i)) - getMinCentralValue());
				
				start = new Point2D.Double(-5, y);
				end = new Point2D.Double(5, y);
				ticks[i] = new Line2D.Double(start, end);
								
			}
			
			return ticks;
					
		}
		
		
		private String[] getYTickLabels() {
			
			
			double lastValue = Double.NEGATIVE_INFINITY;
			
			
			String[] ticks = new String[path.size()];
		
			for (int i = 0; i < ticks.length ; i++ ) {
				
				if (nsg.getCentralProperty(path.get(i)) > lastValue + LayoutConstants.yTickLabelDensityFactor) {
					lastValue = nsg.getCentralProperty(path.get(i));
					ticks[i] = nsg.getCentralPropertyForTable(path.get(i));
				}
				else
					ticks[i] = "";
			}
			
			return ticks;
					
		}
		
		
		private Line2D getXAxis () {
			
			
			
			Point2D start;
			Point2D end;
			start = new Point2D.Double(0, 0);
			
			end = new Point2D.Double(LayoutConstants.nodeXOffset + LayoutConstants.nodeXSpaceFactor * path.size(), 0);
				
			Line2D xAxis = new Line2D.Double(start, end);
			return xAxis;
					
		}
		
		
		
		
		@Override
		public void paint(Graphics g) {
			g.setColor(Color.darkGray);
			Graphics2D g2d = (Graphics2D)g;
			
			Line2D yAxis = getYAxis();
			Line2D[] yticks = getYTicks();
			String[] yTickLabels = getYTickLabels();
			Line2D xAxis = getXAxis();
			
			
			Shape curshape;
			
						
			curshape = vv.getRenderContext().getMultiLayerTransformer().getTransformer(Layer.LAYOUT).transform(yAxis);
			g2d.draw(curshape);
			
			
			curshape = vv.getRenderContext().getMultiLayerTransformer().getTransformer(Layer.LAYOUT).transform(xAxis);
			g2d.draw(curshape);
			
			
			for (int i = 0; i < yticks.length; i++) {
				
				Line2D tick = yticks[i];
				
				curshape = vv.getRenderContext().getMultiLayerTransformer().getTransformer(Layer.LAYOUT).transform(tick);
				
				g2d.draw(curshape);
				
				
				g2d.setFont(new Font("MyFont", Font.PLAIN, LayoutConstants.yTickLabelSize));
				g2d.setPaintMode();
				
				g2d.drawChars(yTickLabels[i].toCharArray(), 0, yTickLabels[i].length(),
						(int) curshape.getBounds2D().getMinX() + LayoutConstants.yTickLabelOffset,
						(int) curshape.getBounds2D().getMinY() + LayoutConstants.yTickLabelSize / 2
					);
			
				
			}
			
			
			
		}

		@Override
		public boolean useTransform() {
			return true;
		}
		
		
	}
	
	
	class Rings implements VisualizationServer.Paintable {

		@Override
		public void paint(Graphics g) {
			
			g.setColor(Color.darkGray);
			Graphics2D g2d = (Graphics2D)g;
			
			Point2D center = vv.getGraphLayout().transform(curCentral);
			Ellipse2D ellipse = new Ellipse2D.Double();
			
			for(double d = LayoutConstants.ringSpaceFactor; d <= LayoutConstants.ringSpaceFactor * LayoutConstants.shellThresholds.length; d+=LayoutConstants.ringSpaceFactor) {
				ellipse.setFrameFromDiagonal(center.getX()-d, center.getY()-d, 
						center.getX()+d, center.getY()+d);
				Shape shape = vv.getRenderContext().getMultiLayerTransformer().getTransformer(Layer.LAYOUT).transform(ellipse);
				g2d.draw(shape);	
				
			}
			
			Point2D start = new Point2D.Double(center.getX(), center.getY() - LayoutConstants.ringSpaceFactor * LayoutConstants.shellThresholds.length);
			Point2D end = new Point2D.Double(center.getX(), center.getY() + LayoutConstants.ringSpaceFactor * LayoutConstants.shellThresholds.length);
			
			g2d.setColor(Color.darkGray);
			Line2D separatorLine = new Line2D.Double(start, end);
			Shape shape = vv.getRenderContext().getMultiLayerTransformer().getTransformer(Layer.LAYOUT).transform(separatorLine);
			g2d.draw(shape);
		}

		@Override
		public boolean useTransform() {
			return true;
		}
		
	}
	
	class CompareByCentralValue implements Comparator<NSG.NSGVertex> {

		@Override
		public int compare(NSGVertex v1, NSGVertex v2) {
			if (v1 == null || v2 == null){
				System.err.println("NULL OCCORED IN PATH!");
				return 0;
			}
			else
				return Double.compare(nsg.getCentralProperty(v1), nsg.getCentralProperty(v2));
		}
		
		
		
	}
	
	
	private ArrayList<NSG.NSGVertex> path = new ArrayList<NSG.NSGVertex>();
		
	private HashSet<NSG.NSGVertex> radial = new HashSet<NSG.NSGVertex>();
	private NSG.NSGVertex curCentral = null;
	
	private ArrayList<NSG.NSGVertex> list = new ArrayList<NSG.NSGVertex>();
	private ArrayList<NSG.NSGVertex> between = new ArrayList<NSG.NSGVertex>();
	private NSG.NSGVertex curLV1 = null;
	private NSG.NSGVertex curLV2 = null;
	
	private VisualizationViewer<NSG.NSGVertex, NSG.NSGEdge> vv;
	
	private NSG nsg;
	private CompareByCentralValue compV = new CompareByCentralValue();
	
	private Axes axes = new Axes();
	private Rings rings = new Rings();
	
	private Graph<NSG.NSGVertex, NSG.NSGEdge> pathGraph = new DirectedSparseGraph<NSGVertex, NSG.NSGEdge>();
	
	
	public PathBuilder(NSG nsg, Collection<NSG.NSGVertex> vs) {
		this.nsg = nsg;
		path.addAll(vs);
		buildGraph();
		
	}
	
	
	
	private double getMinCentralValue() {
		
		return nsg.getCentralProperty(path.get(0));
	}
	
	private double getMaxCentralValue() {
		return nsg.getCentralProperty(path.get(path.size()-1));
	}
	
	
	
	
	public Axes getAxes() {
		return axes;
	}
	
	public Rings getRings () {
		return rings;
	}
	
	public boolean isRadial(NSG.NSGVertex v) {
		return radial.contains(v);
	}
	
	public boolean isCentral(NSG.NSGVertex v) {
		return curCentral.equals(v);
	}
	
	public boolean isList(NSG.NSGVertex v) {
		return list.contains(v);
	}
	
	public boolean isPath(NSG.NSGVertex v) {
		return path.contains(v);
	}
	
	public NSG getNSG() {
		return nsg;
	}
	
	public int getIndex (NSG.NSGVertex v) {
		return path.lastIndexOf(v);
	}
	
	public NSG.NSGVertex getCentral() {
		return curCentral;
	}
	
	
	private void clearRadialInGraph() {
		for (NSG.NSGVertex rv: radial) 
			pathGraph.removeVertex(rv);
	}
	
	private void clearPathInGraph() {
		for (NSG.NSGVertex rv: path) 
			pathGraph.removeVertex(rv);
	}
	
	private void clearListInGraph() {
		for (NSG.NSGVertex rv: list) 
			pathGraph.removeVertex(rv);
	}
	
	
	public void showRadialNeighbors(NSG.NSGVertex v) {
		
		clearListInGraph();
		clearRadialInGraph();
		
		
		//clear current radial set
		this.radial.clear();
		
		
		curCentral = v;
		//refill current radial set with neighbors
		
		for (NSG.NSGVertex vh : nsg.getGraph().getNeighbors(v)) {
			if (compV.compare(vh, curCentral) != 0)
				radial.add(vh);
		}
		
		for (NSG.NSGVertex rv: radial)
			pathGraph.addVertex(rv);
	}
	
	
	
	public boolean shortcut(NSG.NSGVertex v1, NSG.NSGVertex v2) {
		
		int from = path.lastIndexOf(v1);
		int to = path.lastIndexOf(v2);
		
		if (from > to) {
			int h = from;
			from = to;
			to = h;
		}
		
		if ((to-from) == 1)
			return false;
		
		between.clear();
		
		for (int i = from + 1; i < to; i++)
			between.add(path.get(i));
		
		
		clearGraph();
		
		path.removeAll(between);
		buildGraph();
		
		return true;
		
		
	}
	
	private Collection<NSG.NSGVertex> getCloseVertices(NSG.NSGVertex v, Collection<NSG.NSGVertex> col) {
		
		ArrayList<NSG.NSGVertex> disallowed = new ArrayList<NSG.NSGVertex>();
		
		for (NSG.NSGVertex vh : col) {
			if (compV.compare(v,vh) == 0)
				disallowed.add(vh);
		}
			
		return disallowed;
				
		
	}
	
	
	public boolean listPossible(NSG.NSGVertex v1, NSG.NSGVertex v2) {
		//sort vertices
		if (compV.compare(v1, v2) > 0) {
			curLV2 = v1;
			curLV1 = v2;
		}
		else if (compV.compare(v1, v2) < 0) {
			curLV2 = v2;
			curLV1 = v1;
		}
		else if (v1.equals(v2) && path.lastIndexOf(v1) != 0 && path.lastIndexOf(v1) != path.size() - 1) { 
			curLV1 = path.get(path.lastIndexOf(v1) - 1);
			curLV1 = path.get(path.lastIndexOf(v1) + 1);
		}
		else {
			curLV1 = v1;
			curLV2 = v2;
		}
		
		ArrayList<NSG.NSGVertex> templist = new ArrayList<NSG.NSGVertex>();
		
		
		//get successors of v1
		templist.addAll(nsg.getGraph().getSuccessors(v1));
		//join with predecessors of v2
		templist.retainAll(nsg.getGraph().getPredecessors(v2));
		
		templist.removeAll(getCloseVertices(curLV1, templist));
		templist.removeAll(getCloseVertices(curLV2, templist));
		
		
		if (templist.isEmpty())
			return  false;
		
		return true;
	}
	
	
	public void sortVertices(List<NSG.NSGVertex> col) {
		
		Collections.sort(col, compV);
	}
	
	public boolean showList(NSG.NSGVertex v1, NSG.NSGVertex v2) {
		
		//sort vertices
		if (compV.compare(v1, v2) > 0) {
			curLV2 = v1;
			curLV1 = v2;
		}
		else if (compV.compare(v1, v2) < 0) {
			curLV2 = v2;
			curLV1 = v1;
		}
		else if (v1.equals(v2) && path.lastIndexOf(v1) != 0 && path.lastIndexOf(v1) != path.size() - 1) { 
			curLV1 = path.get(path.lastIndexOf(v1) - 1);
			curLV1 = path.get(path.lastIndexOf(v1) + 1);
		}
		else {
			curLV1 = v1;
			curLV2 = v2;
		}
		
		ArrayList<NSG.NSGVertex> templist = new ArrayList<NSG.NSGVertex>();
		
		
		//get successors of v1
		templist.addAll(nsg.getGraph().getSuccessors(v1));
		//join with predecessors of v2
		templist.retainAll(nsg.getGraph().getPredecessors(v2));
		
		
		
		if (templist.isEmpty())
			return false;
		
		clearListInGraph();
		clearRadialInGraph();
		
		list = templist;
		between.clear();
		
		for (int i = path.lastIndexOf(v1) + 1 ; i < path.lastIndexOf(v2); i++)
			between.add(path.get(i));
			
		for (NSG.NSGVertex lv: list)
			pathGraph.addVertex(lv);
		
		return true;
	}	
	
	
	
	private void clearGraph() {
		clearRadialInGraph();
		clearListInGraph();
		clearPathInGraph();
	}
	
	
	
	private void buildGraph() {
		
		Collections.sort(path, compV);
		
		
		
		for (int i = 0; i < path.size(); i++)
			pathGraph.addVertex(path.get(i));
		
		NSG.NSGEdge curEdge;
		for (int i = 0; i < path.size() -1; i++) {
			//System.err.println("i: " + path.get(i).getMol().getExternalId() + ", i+1: " + path.get(i+1).getMol().getExternalId());
		
			//System.err.println("edge: " + nsg.getGraph().findEdge(path.get(i), path.get(i+1)));
			
			curEdge = nsg.getGraph().findEdge(path.get(i), path.get(i+1));
			
			//if (curEdge != null)
				pathGraph.addEdge(curEdge, path.get(i), path.get(i+1));
		}
			
		
	}
	
	
	public void addRadialVertexToPath(NSG.NSGVertex v) {
		
		between.clear();
		
		if (compV.compare(v, curCentral) < 0) {
			
			for (NSG.NSGVertex pv : path) {
				if (compV.compare(pv, curCentral) < 0) {
					between.add(pv);
				}
			}
		}
		else {
			
			for (NSG.NSGVertex pv : path) {
				if (compV.compare(pv, curCentral) > 0) {
					between.add(pv);
				}
			}
			
		}
		
		addVertexToPath( v );
	}
	
	
	
	public void abortRadial() {
		clearGraph();
		radial.clear();
		buildGraph();
	}
	
	public void abortList() {
		clearGraph();
		list.clear();
		buildGraph();
	}
	
	public ArrayList<NSG.NSGVertex> getPathInGraph() {
		ArrayList<NSG.NSGVertex> pig = new ArrayList<NSG.NSGVertex>(path.size());
		
		pig.addAll(path);
		pig.retainAll(pathGraph.getVertices());
		
		return pig;
		
	}
	
	public ArrayList<NSG.NSGVertex> getRadialInGraph() {
		ArrayList<NSG.NSGVertex> pig = new ArrayList<NSG.NSGVertex>(radial.size());
		
		pig.addAll(radial);
		pig.retainAll(pathGraph.getVertices());
		
		return pig;
		
	}
	
	public ArrayList<NSG.NSGVertex> getListInGraph() {
		ArrayList<NSG.NSGVertex> pig = new ArrayList<NSG.NSGVertex>(list.size());
		
		pig.addAll(list);
		pig.retainAll(pathGraph.getVertices());
		
		return pig;
		
	}
	
	
	
	public void addVertexToPath( NSG.NSGVertex v ) {
		
		clearGraph();
		
		radial.clear();
		list.clear();
		
		
		path.removeAll(between);
		
		path.add(v);
		between.clear();
		
		//Collections.sort(path, compV);
		
		curCentral = v;
		
		buildGraph();
				
	}
	
	
	public void removeVerticesFromPath (Collection<NSG.NSGVertex> vs) {
		
		clearGraph();
		
		radial.clear();
		list.clear();
		
		path.removeAll(vs);
		
		buildGraph();
		
	}
	
	
	public double normalizedRankInRadial(NSG.NSGVertex vv, double bottomShellBoundary, double topShellBoundary) {
		
		ArrayList <NSG.NSGVertex> neighbors = new  ArrayList <NSG.NSGVertex>();
		
		double curTc = 0;
		
		boolean isSuc = nsg.getGraph().isSuccessor(vv, curCentral);
		
		//System.err.println("Bottom = " + bottomShellBoundary);
		//System.err.println("Top = " + topShellBoundary);
		
		for (NSG.NSGVertex v: radial){
			
			if (nsg.getGraph().isSuccessor(v, curCentral) != isSuc)
				continue;
			
			curTc = nsg.getTc(curCentral, v);
			
			
			
			if (curTc > bottomShellBoundary && curTc <= topShellBoundary){
				neighbors.add(v);
				//System.err.println("Added: " + v);
			}
		}
		
		Collections.sort(neighbors, compV);
		
		//System.err.println("Size of neighbor set: " + neighbors.size());
		//System.err.println("Last Index: " + neighbors.lastIndexOf(vv));
		
		double size = (double) (neighbors.size() - 1);
		double nRank = (double) neighbors.lastIndexOf(vv);
		
		if (size == 0)
			nRank = 0.5;
		else
			nRank /= size;
		
		double factor = Math.min(size/10 + 0.2, 0.8);
		
		
		
		nRank *= factor;
		nRank += (1-factor)/2;
		
		
		
		//System.err.println("nRank: " + nRank);
		
		nRank *= isSuc ? -1 : 1;
		
		//System.err.println("Normalized Rank in Radial: " + nRank);
		
		
		return nRank;
		
	}
	
	
	public double normalizedRankInList(NSG.NSGVertex vv) {
		
		Collections.sort(list, compV);
		
		return ((double) list.lastIndexOf(vv) / (double) list.size());
		
	}
	
	
	public double getOffsetCentralValue( NSG.NSGVertex v) {
		return nsg.getCentralProperty(v) - getMinCentralValue();
	}
	
	public Graph<NSG.NSGVertex, NSG.NSGEdge> getGraph() {
		return pathGraph;
	}
	
	public NSG.NSGVertex getLV1 () {
		return curLV1;
	}
	
	public NSG.NSGVertex getLV2 () {
		return curLV2;
	}
	
	public int getListSize() {
		return list.size();
	}
	
	public ArrayList<NSG.NSGVertex> getPath() {
		return path;
	}
	
	public void setVisualizationViewer(VisualizationViewer<NSG.NSGVertex, NSG.NSGEdge> vv) {
		this.vv = vv;
	}
	
	
	public Point2D getCenterPoint() {
		
		double x = 0.5 * (double)(LayoutConstants.nodeXOffset + path.size() * LayoutConstants.nodeXSpaceFactor);
		double y = 0.5 * (double)LayoutConstants.nodeYSpaceFactor * getOffsetCentralValue(path.get(path.size()-1));
		
		
		
		return new Point2D.Double(x,y);
		
		
	}
	
	public Point2D getUpperLeftPoint() {
		
		double x = -50d;
		double y = -100.0d + (double)LayoutConstants.nodeYSpaceFactor * getOffsetCentralValue(path.get(path.size()-1));
		
		return new Point2D.Double(x,y);
		
		
	}
	
	public Point2D getLowerRightPoint() {
		
		double x = 50d + (double)(LayoutConstants.nodeXOffset + path.size() * LayoutConstants.nodeXSpaceFactor);;
		double y = 50d;
		
		return new Point2D.Double(x,y);
		
	}
	
	public double getWidth() {
		
		double x = (double)(LayoutConstants.nodeXOffset + path.size() * LayoutConstants.nodeXSpaceFactor);;
		
		return x;
		
	}
	
	
	
}
