/*
 * This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package bit.lsi.sar.nsg.pathbuilder.transformers;

import java.awt.Dimension;
import java.awt.geom.Point2D;

import org.apache.commons.collections15.Transformer;

import edu.uci.ics.jung.algorithms.layout.PolarPoint;

import bit.lsi.sar.nsg.NSG;
import bit.lsi.sar.nsg.NSG.NSGVertex;
import bit.lsi.sar.nsg.pathbuilder.PathBuilder;

public class LayoutInitializer implements Transformer<NSG.NSGVertex, Point2D> {

	private PathBuilder pb;
	
	
	public LayoutInitializer(PathBuilder pb) {
		this.pb = pb;
	}
	
	
	public Dimension getSize() {
		
		double x = LayoutConstants.nodeXOffset + LayoutConstants.nodeXSpaceFactor * pb.getPath().size();
		double y = 20 + LayoutConstants.yOffset + LayoutConstants.nodeYSpaceFactor * (Math.abs(pb.getOffsetCentralValue(pb.getPath().get(0))) + Math.abs(pb.getOffsetCentralValue(pb.getPath().get(pb.getPath().size()-1))));
		
		System.err.println("x : " + x + " y: " + y);
		
		return new Dimension((int)x+1, (int)y+1);
		
	}
	
	
	@Override
	public Point2D transform(NSGVertex v) {
		
		double x = 0;
		double y = 0;
		
		
		
		
		if (pb.isRadial(v)) {
			double sim = pb.getNSG().getTc(pb.getCentral(), v);
			
			//get the shell
			int shell = 1;
			for (double curThreshold : LayoutConstants.shellThresholds) {
				if (sim > curThreshold)
					break;
				shell++;
			}
			
			//System.err.println("Sim = " + sim);
			//System.err.println("Shell = " + shell);
			
			double radius = shell * LayoutConstants.ringSpaceFactor;
			double normRank = pb.normalizedRankInRadial(v, LayoutConstants.shellThresholds[shell-1], shell == 1 ? 1.1: LayoutConstants.shellThresholds[shell-2]);
			
			double theta = (1.0d-normRank) * Math.PI + 1.5d* Math.PI;
			
			Point2D point = PolarPoint.polarToCartesian(theta, radius);
			
			point.setLocation(
					point.getX() + transform(pb.getCentral()).getX(),
					point.getY() + transform(pb.getCentral()).getY());
			
			
			return point;
			
			
		}
		if (pb.isList(v)) {
			
			x = (transform(pb.getLV1()).getX() + transform(pb.getLV2()).getX()) / 2;
			y = LayoutConstants.listYOffset + transform(pb.getLV1()).getY() + LayoutConstants.listSpaceFactor * pb.getListSize() * LayoutConstants.nodeYSpaceFactor * (pb.normalizedRankInList(v) - 0.5);
			
			System.err.println("List y: " + y);
			
			return new Point2D.Double(x, y);
			
		}
		if(pb.isPath(v)) {
			//On the Path
			y = LayoutConstants.yOffset + LayoutConstants.nodeYSpaceFactor * pb.getOffsetCentralValue(v);
			x = (LayoutConstants.nodeXOffset + (double) LayoutConstants.nodeXSpaceFactor /2) + (double) pb.getIndex(v) * LayoutConstants.nodeXSpaceFactor;
			return new Point2D.Double(x, y);
		}
		
		
		return new Point2D.Double();
		
	}

}
