/*
 * This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package bit.lsi.sar.nsg.swing;



import bit.lsi.sar.nsg.*;

public class MolInfo {
	
	private Molecule mol;
	private Target t1;
	private Target t2;
	
	private double selectivity;
	private String selstring = "";
	private boolean hasSelectivity;
	
	
	
	
	
	public MolInfo(Molecule mol, Target t1, Target t2) {
		
		this.mol = mol;
		this.t1 = t1;
		this.t2 = t2;
		
		if (t2 == null) {
			selectivity = 0;
			hasSelectivity = false;
		}
		else {
			hasSelectivity = true;
			selectivity = mol.getpSR(this.t1, this.t2);
			selstring = Potency.pSRtext(selectivity) + 
			" (" + 
			mol.getPotency(t1).toString() 
			+ " : " 
			+ mol.getPotency(t2).toString() + 
			")";
			
		}
		
		
		
	}
	
	public String getTarget1Name() {
		return t1.toString();
	}
	
	public String getTarget2Name() {
		if (hasSelectivity)
			return t2.toString();
		else
			return "";
	}
	
	
	
	public double getPot1 () {
		
		return mol.getPotency(t1).getpKi();
		
	}
	
	public double getPot2 () {
		
		if (hasSelectivity)
			return mol.getPotency(t2).getpKi();
		
		else
			return 0;
		
	}
	
	public String getPot1String() {
		return mol.getPotency(t1).toString();
		
	}
	
	public String getPot2String() {
		if (hasSelectivity)
			return mol.getPotency(t2).toString();
		else
			return "";
		
	}
	
	public double getSelectivity() {
		return selectivity;			
	}
	
	public String getSelectivityString() {
		return selstring;			
	}
	
	public boolean hasSel () {
		return hasSelectivity;
	}
	
	public Target[] getTargets() {
		if (hasSelectivity) {
			Target[] ts = {t1, t2};
			return ts;
		}
		else {
			Target[] ts = {t1};
			return ts;
		}
			
	}
	
	
	
	
	
	
	

}
