/*
 * This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package bit.lsi.sar.nsg.swing;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.MouseListener;
import java.util.ArrayList;

import javax.swing.JInternalFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;

import bit.lsi.sar.nsg.Molecule;
import bit.lsi.sar.nsg.MoleculeSet;
import bit.lsi.sar.nsg.Target;


public class MoleculeInfoFrame extends JInternalFrame {

	/**
	 * 
	 */
	private static final long serialVersionUID = 7022316283575598611L;
	
	
	 //Table with data
    private JScrollPane tableScrollPane;
    
    private JTable potTable;
    private AbstractTableModel potModel;
    
    
    private JTable selTable;
    private AbstractTableModel selModel;
    
    
    private final static String[] potTableColumnNames = {"Target", "Potency"};
    private final static String[] selTableColumnNames = {"Target1", "Target2", "Selectivity"};
    private MolInfo[] potTableRows;
    private MolInfo[] selTableRows;
      
    
    //SmilesDepictor
    private SmilesDepictionPanel sdPanel = new SmilesDepictionPanel();
    
    
    
    
    private void initTable() {
    	
    	potModel = new AbstractTableModel() {

			/**
			 * 
			 */
			private static final long serialVersionUID = -2640689477206754931L;

			@Override
			public int getColumnCount() {
				return potTableColumnNames.length;
			}
			
			@Override
			public String getColumnName(int column) {
				return potTableColumnNames[column];
			}

			@Override
			public int getRowCount() {
				
				return potTableRows.length;
			}

			@Override
			public Object getValueAt(int row, int col) {
				
				
				switch (col) {
				
				case 0: 
					return new TableEntry(
							potTableRows[row].getTarget1Name()
					);
					
				case 1: 
					return new TableEntry(
							potTableRows[row].getPot1(),
							potTableRows[row].getPot1String()
							
					);
					
				default: 
					
					return new TableEntry( 0, "");
						
				}
				
			}
			
			@SuppressWarnings("unchecked")
			@Override
			public Class getColumnClass(int i) {
				return TableEntry.class;
				
			}
		}; //potModel
		
		
		
		
		selModel = new AbstractTableModel() {

			/**
			 * 
			 */
			private static final long serialVersionUID = -2640689477206754931L;

			@Override
			public int getColumnCount() {
				return selTableColumnNames.length;
			}
			
			@Override
			public String getColumnName(int column) {
				return selTableColumnNames[column];
			}

			@Override
			public int getRowCount() {
				
				return selTableRows.length;
			}

			@Override
			public Object getValueAt(int row, int col) {
				
				
				switch (col) {
				
				case 0: 
					return new TableEntry(
							selTableRows[row].getTarget1Name()
					);
					
				case 1: 
					return new TableEntry(
							selTableRows[row].getTarget2Name()
							
					);
					
				
				
				case 2: 
					return new TableEntry(
							selTableRows[row].getSelectivity(),
							selTableRows[row].getSelectivityString()
							);
					
				default: 
					
					return new TableEntry( 0, "");
						
				}
				
			}
			
			@SuppressWarnings("unchecked")
			@Override
			public Class getColumnClass(int i) {
				return TableEntry.class;
				
			}
		}; //selModel
				
		potTable = new JTable(potModel);	
		potTable.setAutoCreateColumnsFromModel(true);
		potTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		
    
    	TableRowSorter<TableModel> potsorter = new TableRowSorter<TableModel>(potModel);
    	potTable.setRowSorter(potsorter);
    	
    	
    	
    	selTable = new JTable(selModel);	
    	selTable.setAutoCreateColumnsFromModel(true);
    	selTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		
    
    	TableRowSorter<TableModel> selsorter = new TableRowSorter<TableModel>(selModel);
    	selTable.setRowSorter(selsorter);
    	
    	
    	
    }
    
    
    private void getMolInfo(Molecule mol) {
    	
      	ArrayList<MolInfo> pmis = new ArrayList<MolInfo>();
      	ArrayList<MolInfo> smis = new ArrayList<MolInfo>();
    	
    	//get all SAR sets
    	for (Target t: mol.getTargets()) {
    		
    		pmis.add(new MolInfo(mol, t, null));
    	}
    	
    	//get all SSR sets
    	
    	for (Target t1: mol.getTargets()) {
			
			for (Target t2: mol.getTargets()) {
				
				if (t1.compareTo(t2) >= 0)
					continue;
			
					MoleculeSet curMs = t1.getMoleculeSet();
					curMs.retainAll(t2.getMoleculeSet());
					
					if (curMs.size() < 5)
						continue;
					
					smis.add(new MolInfo(mol, t1, t2));			
			}
		}
    	
    	
    	selTableRows = new MolInfo[smis.size()];
    	potTableRows = new MolInfo[pmis.size()];
    	
    	smis.toArray(selTableRows);
    	pmis.toArray(potTableRows);
    	   	
    }
    
    
    public void addTableMouseListener (MouseListener ml) {
    	potTable.addMouseListener(ml);
    	selTable.addMouseListener(ml);
    }
    
    public MolInfo getMolInfo () {
    	
    	if (potTable.isFocusOwner())
    		return potTableRows[potTable.convertRowIndexToModel(potTable.getSelectedRow())];
    	else
    		return selTableRows[selTable.convertRowIndexToModel(selTable.getSelectedRow())];
    	
    		
    }
    
    public MoleculeInfoFrame(Molecule mol) {
    	super();
    	
    	getMolInfo(mol);
    	initTable();
    	
    	
    	sdPanel.setPreferredSize(new Dimension(600,300));
    	sdPanel.depictMol(mol);
    	
    	this.getContentPane().add(sdPanel, BorderLayout.NORTH);
    	
    	
    	JPanel tabPanel = new JPanel(new GridLayout(1,2));
    	
    	//Add Table to ScrollPane
		potTable.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
		
		tableScrollPane = new JScrollPane(potTable, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		potTable.setFillsViewportHeight(true);
		tabPanel.add(tableScrollPane);
		
		
		//Add Table to ScrollPane
		selTable.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
		
		tableScrollPane = new JScrollPane(selTable, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		selTable.setFillsViewportHeight(true);
		tabPanel.add(tableScrollPane);
		
		
    	
    	this.getContentPane().add(tabPanel, BorderLayout.CENTER);
    	
    	
    	
    	this.setPreferredSize(new Dimension(600,600));
    	this.setClosable(true);
    	this.setResizable(true);
    	this.setDefaultCloseOperation(JInternalFrame.DISPOSE_ON_CLOSE);
    	
    	
    	this.pack();
    	this.show();
    	
    	
    }
	
	

}
