/*
 * This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package bit.lsi.sar.nsg.swing;

import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Rectangle;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.HashMap;

import javax.swing.JInternalFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.Scrollable;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.RowSorterEvent;
import javax.swing.event.RowSorterListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

import bit.lsi.sar.nsg.Molecule;

public class MoleculeTableFrame extends JInternalFrame {
	
	
	class MyAllPanel extends JPanel implements Scrollable {

		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;

		@Override
		public Dimension getPreferredScrollableViewportSize() {
			// TODO Auto-generated method stub
			return null;
		}

		@Override
		public int getScrollableBlockIncrement(Rectangle visibleRect,
				int orientation, int direction) {
			// TODO Auto-generated method stub
			return 300;
		}

		@Override
		public boolean getScrollableTracksViewportHeight() {
			// TODO Auto-generated method stub
			return false;
		}

		@Override
		public boolean getScrollableTracksViewportWidth() {
			// TODO Auto-generated method stub
			return false;
		}

		@Override
		public int getScrollableUnitIncrement(Rectangle visibleRect,
				int orientation, int direction) {
			// TODO Auto-generated method stub
			return 300;
		}
		
	};
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private JTable connectedTable;
	private MoleculeTableModel tableModel;
	private MyAllPanel allPanel = new MyAllPanel();
	
	private JScrollPane scrollPane;
	private boolean modelHasChanged = true;
	
	private HashMap<String,SmilesDepictionPanel> smilesDepictions = new HashMap<String,SmilesDepictionPanel>();
	
	
	public MoleculeTableFrame(JTable table) {
		super();
		connectedTable = table;
		tableModel = (MoleculeTableModel) connectedTable.getModel();
		
			
		scrollPane = new JScrollPane(allPanel);
		scrollPane.setPreferredSize(new Dimension(320,900));
		
		addListeners();
				
		this.getContentPane().add(scrollPane);
		
		this.setClosable(true);
		this.setDefaultCloseOperation(JInternalFrame.HIDE_ON_CLOSE);
		this.pack();
		//this.show();
		
	}
	
	private void addListeners() {
		
		
		connectedTable.getRowSorter().addRowSorterListener(new RowSorterListener() {
			
			@Override
			public void sorterChanged(RowSorterEvent e) {
				// TODO Auto-generated method stub
				 modelHasChanged = true;
				
			}
		});
		
		
		tableModel.addTableModelListener(new TableModelListener() {
			
			@Override
			public void tableChanged(TableModelEvent e) {
				// TODO Auto-generated method stub
				 modelHasChanged = true;
				 if (tableModel.getRowCount() == 0)
					 updateDepiction();
			}
		});
		
		connectedTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
			
			@Override
			public void valueChanged(ListSelectionEvent e) {
				// TODO Auto-generated method stub
				updateDepiction();
			}
		});
		
		this.addInternalFrameListener(new InternalFrameAdapter(){
			
			@Override
			public void internalFrameActivated(InternalFrameEvent e) {
				updateDepiction();
			}
			
		});
		
		
		
	}
	
	
	private void updateDepiction() {
		
		
		if (this.isVisible()){ 
			
			if (modelHasChanged) {
			
			allPanel.removeAll();
			
				modelHasChanged = false;
			
				int rows = connectedTable.getRowCount();
				
				if (rows > 0 && tableModel.getRowCount()> 0) {
				
							
					
					allPanel.setLayout(new GridLayout(rows, 1));
					allPanel.setPreferredSize(new Dimension(300, 300*rows));
					
					
					int row;
					
					//System.err.println("RowCount (Table): " +  connectedTable.getRowCount());
					//System.err.println("RowCount (Model): " +  tableModel.getRowCount());
					
					
					for (int i = 0; i < connectedTable.getRowCount(); i++) {
						
						row = connectedTable.convertRowIndexToModel(i);
						
						//System.err.println("i: " + i + ", row: "+ row);
						
						allPanel.add(getSmilesDepictionPanel( tableModel.getMoleculeAt(row), i ));
					}
				
				}
				
				this.revalidate();
			}	
				//allPanel.scrollRectToVisible(new Rectangle(0, 300*connectedTable.getSelectedRow(), 300, 900));
				//allPanel.revalidate();
				//allPanel.repaint();
				
				this.repaint();
			
							
		}
			
		
	}
	
	private SmilesDepictionPanel getSmilesDepictionPanel( Molecule mol, final int i ) {
		
		
		SmilesDepictionPanel sdp =  smilesDepictions.get(mol.getSmiles());
		if (sdp == null) {
			sdp = new SmilesDepictionPanel(mol);
			smilesDepictions.put(mol.getSmiles(), sdp);
		}
		
		sdp.setPreferredSize(new Dimension(250,300));
		
		sdp.addMouseListener(new MouseAdapter() {
			
			@Override
			public void mouseClicked(MouseEvent e) {
				connectedTable.getSelectionModel().setSelectionInterval(i, i);
				
			}
		});
		
		return sdp;
		
	}
	

}
