/*
 * This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */


/**
 * 
 */
package bit.lsi.sar.nsg.swing;

import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.Paint;
import java.awt.Point;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Point2D;
import java.awt.image.BufferedImage;
import java.beans.PropertyVetoException;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.ConcurrentModificationException;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;

import javax.imageio.ImageIO;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDesktopPane;
import javax.swing.JFileChooser;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JToolTip;
import javax.swing.ListSelectionModel;
import javax.swing.ToolTipManager;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;

import org.apache.commons.collections15.Predicate;
import org.apache.commons.collections15.Transformer;

import au.com.bytecode.opencsv.CSVWriter;
import bit.lsi.sar.nsg.Molecule;
import bit.lsi.sar.nsg.NSG;
import bit.lsi.sar.nsg.SARSet;
import bit.lsi.sar.nsg.NSG.NSGEdge;
import bit.lsi.sar.nsg.NSG.NSGVertex;
import bit.lsi.sar.nsg.mousemenus.NSGMouseMenuManager;
import bit.lsi.sar.nsg.pathbuilder.transformers.LayoutConstants;
import bit.lsi.sar.nsg.pathbuilder.transformers.MutablePickedState;
import edu.uci.ics.jung.algorithms.layout.GraphElementAccessor;
import edu.uci.ics.jung.algorithms.layout.ISOMLayout;
import edu.uci.ics.jung.algorithms.layout.Layout;
import edu.uci.ics.jung.algorithms.shortestpath.DijkstraShortestPath;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.util.Context;
import edu.uci.ics.jung.visualization.DefaultVisualizationModel;
import edu.uci.ics.jung.visualization.GraphZoomScrollPane;
import edu.uci.ics.jung.visualization.RenderContext;
import edu.uci.ics.jung.visualization.VisualizationImageServer;
import edu.uci.ics.jung.visualization.VisualizationModel;
import edu.uci.ics.jung.visualization.VisualizationViewer;
import edu.uci.ics.jung.visualization.control.CrossoverScalingControl;
import edu.uci.ics.jung.visualization.control.GraphMouseListener;
import edu.uci.ics.jung.visualization.control.PickingGraphMousePlugin;
import edu.uci.ics.jung.visualization.control.PluggableGraphMouse;
import edu.uci.ics.jung.visualization.control.SatelliteVisualizationViewer;
import edu.uci.ics.jung.visualization.control.ScalingGraphMousePlugin;
import edu.uci.ics.jung.visualization.control.TranslatingGraphMousePlugin;
import edu.uci.ics.jung.visualization.picking.PickedInfo;
import edu.uci.ics.jung.visualization.renderers.BasicEdgeRenderer;
import edu.uci.ics.jung.visualization.renderers.BasicRenderer;
import edu.uci.ics.jung.visualization.renderers.Renderer;


/**
 * @author lounkine
 *
 */
public class NSGFrame extends AbstractNSGFrame {

	/**
	 * 
	 */
	private static final long serialVersionUID = 4981161613981295031L;
	
	private NSG nsg;
	private JDesktopPane parentPane;
	
	private boolean showEdgesOnInit = true;
	
	private abstract class SwitchableItemListener implements ItemListener {
		
		protected boolean on = true;
				
		public void switchOn() {
			on = true;
			tableModel.fireTableDataChanged();
		}
		
		public void switchOff() {
			on = false;
		}
		
	};
	
	private SwitchableItemListener pickItemListener;
	
	class MyISOMLayout extends ISOMLayout<NSG.NSGVertex, NSG.NSGEdge> {

		private boolean mute = false;
		
		public MyISOMLayout(Graph<NSGVertex, NSGEdge> g) {
			super(g);
		}
		
		@Override
		public boolean done() {
			if (mute)
				return true;
			else if (super.done()) {
				muteAndFinalize();
				return true;
			}
			else
				return false;
				
		}
		
		public void mute(boolean m) {
			this.mute = m;
		}
		
		
		public synchronized void muteAndFinalize() {
			this.mute = true;
			
			Point2D curPos;
			
			for (NSG.NSGVertex v: this.getGraph().getVertices()) {
				
				curPos = this.transform(v);
				curPos.setLocation(curPos.getX() + Math.random()*50, curPos.getY()+ Math.random()*50);
				
				this.setLocation(v, curPos);
				
			}
			
		}
		
		
		
		
		
		
	}
	
	
	//Graph and Layout
	private Graph<NSG.NSGVertex, NSG.NSGEdge> nsGraph;
	private MyFRLayout layout;
	
	DijkstraShortestPath<NSGVertex,NSGEdge> sp;
	
	
	
	//The model that the two viewers will share
	private VisualizationModel<NSG.NSGVertex, NSG.NSGEdge> vm;
	
	// the preferred sizes for the two views
    private static Dimension preferredSize1 = new Dimension(400,400);
    private static Dimension preferredSize2 = new Dimension(250, 250);
    
    //The two viewers
    private VisualizationViewer<NSG.NSGVertex, NSG.NSGEdge> vv1;	//The main Viewer
    private SatelliteVisualizationViewer<NSG.NSGVertex, NSG.NSGEdge> vv2;	//The satellite Viewer
    
    private CrossoverScalingControl coScaler = new CrossoverScalingControl();
    
    
    //An edge renderer that does not do anything
    private BasicEdgeRenderer<NSG.NSGVertex, NSG.NSGEdge> lazyRenderer = new BasicEdgeRenderer<NSG.NSGVertex, NSG.NSGEdge>() {
		
		@Override
		public void paintEdge(RenderContext<NSG.NSGVertex, NSG.NSGEdge> rc, Layout<NSG.NSGVertex, NSG.NSGEdge> l, NSG.NSGEdge e)
		{
			//don't paint
		}
		
		@Override
		protected void drawSimpleEdge(RenderContext<NSG.NSGVertex, NSG.NSGEdge> rc, Layout<NSG.NSGVertex, NSG.NSGEdge> l, NSG.NSGEdge e)
		{
			//don't paint
		}
		
		
		
	};
	
	//An edge renderer that should work properly
	private BasicEdgeRenderer<NSG.NSGVertex, NSG.NSGEdge> defaultRenderer = new BasicEdgeRenderer<NSG.NSGVertex, NSG.NSGEdge>();
    
    //panels
    private Container nsgPanel  = new JPanel(new BorderLayout());
    private Container rightPanel = new JPanel(new GridLayout(2,1));
    private Container lowerPanel = new JPanel(new GridLayout(1,1));
    private Container infoPanel = new JPanel(new GridLayout(4,1));
    private Container tablePanel = new JPanel(new BorderLayout());
    
    
    private final JFileChooser saveCSVChooser = new JFileChooser();
   
    private MoleculeTableFrame molTableFrame;
    private MutablePickedState sharedPickedState = new MutablePickedState();
    
    //Table with Molecules
    private JScrollPane tableScrollPane;
    private JTable moleculeTable;
    private AbstractTableModel tableModel;
    
    private JButton bExportCSV = new JButton("Export CSV");
    private JButton bLeaveSelection = new JButton("Drop below selected");
    private JButton bSavePNG = new JButton("Save graph to PNG");
    
    private JButton bShowMolList = new JButton("Show selected structures");
    private JButton bCalcPath = new JButton ("Find path");
    private JButton bPathBuilder = new JButton ("Edit path");
    
    
    
    //Additional Data Structures needed for the Table
    private String[] tableColumnNames;
    private LinkedList<NSG.NSGVertex> tableRows = new LinkedList<NSG.NSGVertex>();
    
       
    //Stuff in the infoPanel
    private JCheckBox cbShowEdges;
    private JCheckBox cbShowArrows;
    private JTextField tfExternalId;
    
    private String[] locals = new String[3]; 
    private JLabel lScoreCont = new JLabel("Cont.: ");
    private JLabel lScoreDisc = new JLabel("Disc.: ");
    private JLabel lScoreSARI = new JLabel("SARI: ");
    private JLabel lNumberMolecules = new JLabel (" Molecules");
    
    private JButton bSynchro = new JButton("Synchronize Selection");
    //private JButton bAllSAR = new JButton("All SAR sets");
    //private JButton bAllSSR = new JButton("All SSR sets");
    
    
    
    
    
    
    //mouse menu listeners
    private final ActionListener mmListenerSarTree = new ActionListener() {
    	
    	@Override
		public void actionPerformed(ActionEvent e) {
			
			if ( getParentFrame().equals(NSGMouseMenuManager.sharedInstance().getActiveNSGFrame()) ) {
				makeSTF(NSGMouseMenuManager.sharedInstance().menuItemSarTree.getVertex());
			}
			
		}
    	
    	
    };
    
    private final ActionListener mmListenerSelectNeighbors = new ActionListener() {

		@Override
		public void actionPerformed(ActionEvent e) {
			
			if (getParentFrame().equals(NSGMouseMenuManager.sharedInstance().getActiveNSGFrame()) ) {
				selectNeighbors(NSGMouseMenuManager.sharedInstance().menuItemSelectNeighbors.getVertex());
			}
			
		}
    	
    };
    
    
    
    
    
    
    
    private void makeColorful() {
	
		vv1.setBackground(Color.white);
		
		
		final Comparator<NSG.NSGVertex> compPicked = new Comparator<NSG.NSGVertex> () {

			@Override
			public int compare(NSGVertex v1, NSGVertex v2) {
				
				
				if (moleculeTable.getSelectedRow() != -1 && tableRows.get(moleculeTable.convertRowIndexToModel(moleculeTable.getSelectedRow())).equals(v1))
					return 1;
				else if ((moleculeTable.getSelectedRow() != -1 && tableRows.get(moleculeTable.convertRowIndexToModel(moleculeTable.getSelectedRow())).equals(v2)))
					return -1;
				
				double h1 = 0;
				double h2 = 0;
				
				if (vv1.getPickedVertexState().isPicked(v1)){
				
					
					if (vv1.getPickedVertexState().isPicked(v2)) {
						//Both Picked. look which one is bigger
						h1 = vv1.getRenderContext().getVertexShapeTransformer().transform(v1).getBounds().getHeight();
						h2 = vv1.getRenderContext().getVertexShapeTransformer().transform(v2).getBounds().getHeight();
						
						
						if (h1 > h2)
							return -1;
						else if (h1 < h2)
							return 1;
						else
							return 0;
								
					}
					else {
						return 1;
					}
				}
				else {
					if (vv1.getPickedVertexState().isPicked(v2)) {
						return -1;
					}
					else {
						h1 = vv1.getRenderContext().getVertexShapeTransformer().transform(v1).getBounds().getHeight();
						h2 = vv1.getRenderContext().getVertexShapeTransformer().transform(v2).getBounds().getHeight();
						
						
						if (h1 > h2)
							return -1;
						else if (h1 < h2)
							return 1;
						else
							return 0;
					}
				
				}
			}
			
		};
		
		Renderer<NSG.NSGVertex, NSG.NSGEdge> renderer = new BasicRenderer<NSG.NSGVertex, NSG.NSGEdge> () {
			
			
			@Override
			public void render(RenderContext<NSG.NSGVertex, NSG.NSGEdge> renderContext, Layout<NSG.NSGVertex, NSG.NSGEdge> layout) {
				
				// paint all the edges
		        try {
		        	for(NSG.NSGEdge e : layout.getGraph().getEdges()) {

				        renderEdge(
				                renderContext,
				                layout,
				                e);
				        renderEdgeLabel(
				                renderContext,
				                layout,
				                e);
		        	}
		        } catch(ConcurrentModificationException cme) {
		        	renderContext.getScreenDevice().repaint();
		        }
				
				// paint all the vertices
		        
		        
		        ArrayList<NSG.NSGVertex> prioritizedVertices = new ArrayList<NSG.NSGVertex>();
		        
		        prioritizedVertices.addAll(layout.getGraph().getVertices());
		        
		        Collections.sort(prioritizedVertices, compPicked);
		        
		        
		        
		        
		        try {
		        	for(NSG.NSGVertex v : prioritizedVertices) {

				    	renderVertex(
				                renderContext,
		                        layout,
				                v);
				    	renderVertexLabel(
				                renderContext,
		                        layout,
				                v);
		        	}
		        } catch(ConcurrentModificationException cme) {
		            renderContext.getScreenDevice().repaint();
		        }
			}
			
			
		};

		Predicate<Context<Graph<NSG.NSGVertex, NSG.NSGEdge>, NSG.NSGEdge>> apredicate = new Predicate<Context<Graph<NSG.NSGVertex, NSG.NSGEdge>, NSG.NSGEdge>> () {

			@Override
			public boolean evaluate(
					Context<Graph<NSGVertex, NSGEdge>, NSGEdge> c) {
				// TODO Auto-generated method stub
				return (!(cbShowArrows.getSelectedObjects() == null)
						&& !c.element.connectsAdd()
						&& !c.element.hasZeroDiff()
						);
			}
			
		};
		
		
		//Set up a Vertex to Paint Transformer
		Transformer<NSG.NSGVertex,Paint> vertexPaint = new Transformer<NSG.NSGVertex,Paint>() {
			public Paint transform(NSG.NSGVertex v) {
					
				if (v.isInSarSet()) {
					
					//System.err.println("The color value is " + v.getColorValue());
					float hue = (float) (0.333 * (1-v.getColorValue()));
					return Color.getHSBColor(hue, 1.0f, 1.0f);
					
				}
				else
				{
					return Color.darkGray;
				}
					
			}
			
		};
		
		
		Transformer<NSG.NSGVertex,Paint> vertexBorderPaintTransformer = new Transformer<NSG.NSGVertex,Paint>() {
			public Paint transform(NSG.NSGVertex v) {
					
				if (isVertexSelectedInTable(v))
					return Color.magenta;
					
				
				else if (vv1.getPickedVertexState().isPicked(v))
					return Color.blue;
				else
					return v.getSetColor();
					
			}
			
		};
		
		
		//Set up a Vertex to Shape Transformer
		Transformer<NSG.NSGVertex,Shape> vertexShape = new Transformer<NSG.NSGVertex,Shape>() {
			public Shape transform(NSG.NSGVertex v) {
					
				double x,h;
								
				//if (v.isInSarSet()) {
					
					h = 10 + 40 * v.getSizeValue();
					
				//}
				//else
				//{
				//	h = 10;
										
				//}
				
				x = -h/2;
				
				return new Ellipse2D.Double(x,x,h,h);
					
			}
			
		};
		
		
		//Set up a vertex stroke transformer
		Transformer<NSG.NSGVertex, Stroke> vertexStrokeTransformer = new Transformer<NSG.NSGVertex, Stroke> () {

			@Override
			public Stroke transform(NSG.NSGVertex v) {
				
					int addVal = 0;
				
					if(isVertexSelectedInTable(v))
						addVal = 10;
						
				
				
				
					if (v.isInAddSet()) 
						return new BasicStroke(5+addVal);
					else if (vv1.getPickedVertexState().isPicked(v))
						return new BasicStroke(3+addVal);
					else
						return new BasicStroke(0);
				
				
			}
			
			
			
		};
		
		
		//Set up an edge Transformer
		Transformer<NSG.NSGEdge, Paint> edgeColor = new Transformer<NSG.NSGEdge, Paint> () {
			public Paint transform(NSG.NSGEdge e) {
				
				
				
				
				float[] hsb = new float[3];
				
				boolean isP = vv1.getPickedEdgeState().isPicked(e);
				NSG.NSGVertex v = e.getAddVertex();
				double lower = v.isInAddSet() ?  v.getThreshold(): SARSet.getStaticCutoff();
				
				
				final double mv = 0.8;	//make visible
				
				
				if (isP) {
					Color.RGBtoHSB(0,0,255, hsb);
									
					hsb[1] = (float) ( 
							( e.getSim() - lower )
							/ (1 - lower)
							);
					
					
					hsb[2] = 1;
					hsb[1] = (float) (mv * hsb[1] + (1-mv));
				}
				else if (v.isInAddSet()){
					Color vcol = v.getSetColor();
					Color.RGBtoHSB(vcol.getRed(), vcol.getGreen(), vcol.getBlue(), hsb);
					
					hsb[1] = (float) ( 
							( e.getSim() - lower )
							/ (1 - lower)
							);
					
					hsb[2] = 1;
					hsb[1] = (float) (mv * hsb[1] + (1-mv));
				}
				else {
					hsb[0] = 0;
					hsb[2] = (float) (1 -   
							( e.getSim() - lower )
							/ (1 - lower)
							);
					hsb[1] = 0;
					hsb[2] = (float) (mv * hsb[2] );
					
				}
				
				
				
				return Color.getHSBColor(hsb[0], hsb[1], hsb[2]);

			}
		};
		
		
		/*
		PickableVertexPaintTransformer<NSG.NSGVertex> pickVertex = new  PickableVertexPaintTransformer<NSG.NSGVertex>(
			
				vv1.getPickedVertexState(),
				Color.black,
				Color.blue
		
			
		);*/
		
		
		
		vv1.setRenderer(renderer);
		vv2.setRenderer(renderer);
		
		
		vv1.getRenderContext().setVertexFillPaintTransformer(vertexPaint);
		vv2.getRenderContext().setVertexFillPaintTransformer(vertexPaint);
		
		vv1.getRenderContext().setVertexDrawPaintTransformer(vertexBorderPaintTransformer);
		vv2.getRenderContext().setVertexDrawPaintTransformer(vertexBorderPaintTransformer);
		
		vv1.getRenderContext().setVertexStrokeTransformer(vertexStrokeTransformer);
		vv2.getRenderContext().setVertexStrokeTransformer(vertexStrokeTransformer);
		
		vv1.getRenderContext().setVertexShapeTransformer(vertexShape);
		vv2.getRenderContext().setVertexShapeTransformer(vertexShape);
		
		vv1.getRenderContext().setEdgeDrawPaintTransformer(edgeColor);
		vv2.getRenderContext().setEdgeDrawPaintTransformer(edgeColor);
		
		vv1.getRenderContext().setEdgeArrowPredicate(apredicate);
		vv2.getRenderContext().setEdgeArrowPredicate(apredicate);
		
		
		vv1.setVertexToolTipTransformer(new ToolTipShowMol());
		vv1.setEdgeToolTipTransformer(new ToolTipForEdge());
		
		
		vv1.getRenderer().setEdgeRenderer(defaultRenderer);
		vv2.getRenderer().setEdgeRenderer(defaultRenderer);
		
		
				
	}
	
    
    private boolean  makePB() {
    	return makePB(tableRows);
    }
	
    protected boolean makePB(Collection<NSG.NSGVertex> col) {
    	
    	final PathBuilderFrame pbf;
    	
    	try {
    		pbf = new PathBuilderFrame(vv1, nsg, col, this);
    	}
    	catch (IllegalArgumentException e) {
    		e.printStackTrace();
    		return false;
    	}
    	
    	parentPane.add(pbf);
		pbf.toFront();
		
		moleculeTable.addKeyListener(new KeyAdapter() {

			@Override
			public void keyReleased(KeyEvent arg0) {
				// TODO Auto-generated method stub
				pbf.repaint();
			}
			
		});
		
		
		moleculeTable.addMouseListener(new MouseAdapter() {
		
			@Override
			public void mouseClicked(MouseEvent e) {
				pbf.repaint();
			}
			
			
		});
		
		moleculeTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
			
			@Override
			public void valueChanged(ListSelectionEvent e) {
				pbf.repaint();
				
			}
		});
		
		this.addInternalFrameListener(new InternalFrameAdapter() {

			@Override
			public  void internalFrameClosed(InternalFrameEvent e) {
				pbf.dispose();
			}
			
			
			
		});
    	
		return true;
    }
    
    private void makeSTF(NSG.NSGVertex v) {
    	
    	final SARTreeFrame stf = new SARTreeFrame(this, vv1, nsg, v);
		parentPane.add(stf);
		stf.toFront();
		
		moleculeTable.addKeyListener(new KeyAdapter() {

			@Override
			public void keyReleased(KeyEvent arg0) {
				// TODO Auto-generated method stub
				stf.repaint();
			}
			
		});
		
		
		moleculeTable.addMouseListener(new MouseAdapter() {
		
			@Override
			public void mouseClicked(MouseEvent e) {
				stf.repaint();
			}
			
			
		});
		
		moleculeTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
			
			@Override
			public void valueChanged(ListSelectionEvent e) {
				stf.repaint();
				
			}
		});
		
		this.addInternalFrameListener(new InternalFrameAdapter() {

			@Override
			public  void internalFrameClosed(InternalFrameEvent e) {
				stf.dispose();
			}
			
			
			
		});
		
		
    }
    
    
    private void selectNeighbors(NSG.NSGVertex v) {
    	Collection<NSG.NSGVertex> neighbors = nsGraph.getNeighbors(v);
		
		for (NSG.NSGVertex vv : nsGraph.getVertices())
		{
			if (v.equals(vv))
				continue;
			
			vv1.getPickedVertexState().pick(vv, 
					(neighbors.contains(vv))
			);
		}
		
		vv1.repaint();
		vv2.repaint();
		
    }
    
    
    private boolean  pickPath() {
    	
    	if (tableRows.size()< 2)
    		return false;
    	
    	List<NSG.NSGEdge> edgeList;
		if ( nsg.getCentralProperty(tableRows.getFirst()) > nsg.getCentralProperty(tableRows.get(1)) )
			edgeList = sp.getPath(tableRows.get(1), tableRows.getFirst());
		
		else 
			edgeList = sp.getPath(tableRows.getFirst(), tableRows.get(1));
		
		
		if (edgeList.size() <= 2)
			return false;
		
		
		pickItemListener.switchOff();
		
		vv1.getPickedEdgeState().clear();
		vv1.getPickedVertexState().clear();
		
		pickItemListener.switchOn();
		
		vv1.getPickedVertexState().pick(nsg.getGraph().getSource(edgeList.get(0)), true);
		
		for (NSG.NSGEdge eh : edgeList) {
			vv1.getPickedEdgeState().pick(eh, true);
			
			NSG.NSGVertex vh1 = nsg.getGraph().getDest(eh);
			
			if ( vh1 != null)
				vv1.getPickedVertexState().pick(vh1, true);
			
			vh1 = nsg.getGraph().getSource(eh);
			if (vh1 != null)
				vv1.getPickedVertexState().pick(vh1, true);
			
		}
		
		return true;
		
			
			
		
		
		
    }
    
	private void initgraph() {
		
		
		sp = new DijkstraShortestPath<NSGVertex,NSGEdge>(nsg.getGraph(), new Transformer<NSGEdge, Number>() {
			
			@Override
			public Number transform(NSGEdge e) {
				// TODO Auto-generated method stub
				return e.getFancy();
			}
			
		});
		
		
		this.molToolTip = new MoleculeToolTip(sdPanel, nsg);
		
		
		
		int lDim = (int) ( Math.log(nsGraph.getVertexCount()) * (double)LayoutConstants.graphLayoutNodeSpacingFactor);
		layout.setSize(new Dimension(lDim,lDim));
		
		layout.setRepulsionMultiplier(LayoutConstants.graphLayoutRepulsionMultiplyer);
		
		//((FRLayout2<NSG.NSGVertex, NSG.NSGEdge>) layout).setRepulsionMultiplier(LayoutConstants.graphLayoutRepulsionMultiplyer);
		
		
		//Init the common visualization model
		vm = new DefaultVisualizationModel<NSG.NSGVertex, NSG.NSGEdge>(layout, preferredSize1);
		
		// create 2 views that share the same model
        vv1 = new VisualizationViewer<NSG.NSGVertex, NSG.NSGEdge>(vm, preferredSize1) {

			/**
			 * 
			 */
			private static final long serialVersionUID = 1L;
			
			@Override
			public JToolTip createToolTip() {
				
				if (overVertex)
					return molToolTip;
				else
					return new JToolTip();
			}
			
			
			
		};
    
        
        vv2 = new SatelliteVisualizationViewer<NSG.NSGVertex, NSG.NSGEdge>(vv1, preferredSize2);
        
        vv1.setPickedVertexState(sharedPickedState);
        
        //Set Transformers
        makeColorful ();
        
        
       if (!showEdgesOnInit)
       {
        
    	   vv1.getRenderer().setEdgeRenderer(lazyRenderer);
    	   vv2.getRenderer().setEdgeRenderer(lazyRenderer);
    	   
       }
        
       
       //coScaler.setCrossover(0.5d);
       
        //Add Scaling Control
        vv1.scaleToLayout(coScaler);
        vv2.scaleToLayout(coScaler);
                
        // create a GraphMouse for the main view
        PluggableGraphMouse graphMouse = new PluggableGraphMouse();
        
        //Make the wheel do the navigating
        graphMouse.add(new TranslatingGraphMousePlugin(MouseEvent.BUTTON2_MASK));
        graphMouse.add(new ScalingGraphMousePlugin(coScaler, 0, 1.1f, 0.9f));
        
        //The left button is used to pick things. If shift is pressed then we add to the selection
        graphMouse.add(new PickingGraphMousePlugin<NSG.NSGVertex,NSG.NSGEdge>(MouseEvent.BUTTON1_MASK, MouseEvent.BUTTON1_MASK | MouseEvent.SHIFT_MASK));
        graphMouse.add(NSGMouseMenuManager.sharedInstance().getMousePlugin());
        //Add the mouse to the main view
        vv1.setGraphMouse(graphMouse);
        
        
        //Add functionality to select neighbors and edges after double click
        vv1.addGraphMouseListener( new GraphMouseListener<NSG.NSGVertex>() {

			@Override
			public void graphClicked(NSGVertex v, MouseEvent me) {
				
			
				//System.err.println(me.getButton());
				
				if (me.getButton() == MouseEvent.BUTTON1){
					
					if (me.getClickCount() == 2)
						selectNeighbors(v);
					
				}
							
			}

			@Override
			public void graphPressed(NSGVertex v, MouseEvent me) {
				
				
			}

			@Override
			public void graphReleased(NSGVertex v, MouseEvent me) {
				
				
			}
       	
        });
        
        
       
        
        vv1.addKeyListener(new KeyListener() {

			@Override
			public void keyPressed(KeyEvent e) {
				
								
			}

			@Override
			public void keyReleased(KeyEvent e) {
				
				//TODO: Implement ctrl-a!!
				if (e.getKeyCode() == 0 && e.getModifiers() == 2 && e.getModifiersEx() == 128) { //Ctrl-a 
						
						for (NSG.NSGVertex v : nsGraph.getVertices())
							vv1.getPickedVertexState().pick(v, true);
				}
			}

			
			
			@Override
			public void keyTyped(KeyEvent e) {
				
				if (e.getKeyChar() == KeyEvent.VK_ESCAPE) {
					
					pickItemListener.switchOff();
					
					vv1.getPickedVertexState().clear();
					vv1.getPickedEdgeState().clear();
					
					pickItemListener.switchOn();
					
					layout.abort();
					
					vv1.repaint();
					vv2.repaint();
					
				}			
				
				else if (e.getKeyChar() == KeyEvent.VK_T ) {
					if (!tableRows.isEmpty()) {
						
						makeSTF(tableRows.getFirst());
					}
					
					
					
				}
				
				else if (e.getKeyChar() == KeyEvent.VK_B ) {
					if (!tableRows.isEmpty()) {
						
						if (!makePB())
							JOptionPane.showMessageDialog(e.getComponent(), "Selected molecules do not form a valid path!", "No Path!", JOptionPane.ERROR_MESSAGE);
					}
					
					
				}
				
				else if (e.getKeyChar() == KeyEvent.VK_P ) {
					
						if (!pickPath())
							JOptionPane.showMessageDialog(e.getComponent(), "No Path found!", "No Path!", JOptionPane.ERROR_MESSAGE);
							
								
				}
				
			}	
        });
    
       
       
        pickItemListener = new SwitchableItemListener() {

			@Override
			public void itemStateChanged(ItemEvent e) {
				
			
				
				if (e.getStateChange() == ItemEvent.DESELECTED)
					tableRows.remove(e.getItem());
				else
					tableRows.add((NSGVertex) e.getItem());
					
					if (this.on) 
						updateTable();
				}
        	
        };  
        
       
        vv1.getPickedVertexState().addItemListener( pickItemListener );
        
        
        
        
        
        
	}
	
	
	private void initInfoPanel() {
		
		
		locals = nsg.getGlobalScores();
		
		lScoreCont.setText("Cont.: " + locals[0] );
		lScoreDisc.setText("Disc.: " + locals[1] );
		lScoreSARI.setText("SARI.: " + locals[2] );
		lNumberMolecules.setText(nsg.getNumberMolecules()  + " Molecules");
		
		JPanel globalScoresPanel = new JPanel(new GridLayout(2,2));
		
		globalScoresPanel.add(lNumberMolecules);
		globalScoresPanel.add(lScoreSARI);
		globalScoresPanel.add(lScoreCont);
		globalScoresPanel.add(lScoreDisc);
	
		
		infoPanel.add(globalScoresPanel);
		
		
		cbShowEdges = new JCheckBox("Show Edges", showEdgesOnInit);
		cbShowArrows = new JCheckBox("Show Arrows", false);
		
		//Add a listener that will toggle the vv1 and vv2 states
		cbShowEdges.addItemListener(new ItemListener () {

			@Override
			public void itemStateChanged(ItemEvent e) {
				
				if (e.getStateChange() == ItemEvent.DESELECTED) {
					
					vv1.getRenderer().setEdgeRenderer(lazyRenderer);
					vv2.getRenderer().setEdgeRenderer(lazyRenderer);
								
					
				}
				else {
					
					vv1.getRenderer().setEdgeRenderer(defaultRenderer);
					vv2.getRenderer().setEdgeRenderer(defaultRenderer);
					
				}
				
				vv1.repaint();
				vv2.repaint();
				
			}
			
		});
		
		cbShowArrows.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				
				vv1.repaint();
				vv2.repaint();
				
			}
		});
		
		
		JPanel cbPanel = new JPanel(new GridLayout(1,2));
		
		cbPanel.add(cbShowEdges);
		cbPanel.add(cbShowArrows);
		infoPanel.add(cbPanel);
		
		
		tfExternalId = new JTextField(10);
		
		//Add Text change listener
		tfExternalId.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				
				selectBasedOnExternalId(tfExternalId.getText());
			}
			
		});
		
		JPanel buttonPanel = new JPanel(new GridLayout(3,1));
		buttonPanel.add(bSynchro);
		//buttonPanel.add(bAllSAR);
		//buttonPanel.add(bAllSSR);
		
		infoPanel.add(buttonPanel);
		
		JPanel textInputPanel = new JPanel(new GridLayout(2,1));
		textInputPanel.add(new JLabel("Search for Id:"));
		textInputPanel.add(tfExternalId);
		
		infoPanel.add(textInputPanel);
		
		
		
	}
	
	
	private void initRightPanel() {
		
		rightPanel.add(vv2);
		
		initInfoPanel();
		rightPanel.add(infoPanel);
		
       
        
	}
	
	
	protected void updateTable() {
		
		
		//updateTableRows();
		
		tableModel.fireTableDataChanged();
		tableSelectionChange();
	}
	
	
	
	
	private void tableSelectionChange() {
			
		vv1.repaint();
		vv2.repaint();		
	}
	
	
	private boolean saveAsPNG() {
		
		//System.err.println("saveASPNG");		
		
		
		int returnVal = saveCSVChooser.showSaveDialog(this);
		
		if (returnVal != JFileChooser.APPROVE_OPTION) 
			return false;
		
		File file = saveCSVChooser.getSelectedFile();
		
		BufferedImage im = new BufferedImage(vv1.getWidth(), vv1.getHeight(), BufferedImage.TYPE_INT_RGB);
		Graphics2D graphics = im.createGraphics();
		
		graphics.setRenderingHints(vv1.getRenderingHints());
        vv1.paint(graphics);
        graphics.dispose();
		
		try {
			ImageIO.write(im, "png", file);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}
		
		return true;
		
	}
	
	
	private void initTablePanel() {
		
		//Init the Table Model
		
		tableModel = new MoleculeTableModel(tableColumnNames, nsg, tableRows);
			
			
		//Init the Table
		moleculeTable = new JTable(tableModel) {
					
			private static final long serialVersionUID = 1L;

			private int lastRow = -1;
			private int lastCol = -1;
			private Point lastPoint = null;
			
			
			
			@Override
			public JToolTip createToolTip() {
								
				return molToolTip;
			}
			
			@Override
			public String getToolTipText(MouseEvent e) {
				
				Point p = e.getPoint();
				
				int rowIndex = rowAtPoint(p);
				
				if (rowIndex >= 0 && getRowCount() > 0) {
					int row = convertRowIndexToModel(rowIndex);
					NSG.NSGVertex v = tableRows.get(row);
					sdPanel.depictMol(v.getMol());
					ToolTipManager.sharedInstance().setEnabled(true);
					return v.getMol().toString();
				}
				else {
					
					ToolTipManager.sharedInstance().setEnabled(false);
					return "";
				}
				
			}
			
			@Override
			public Point getToolTipLocation(MouseEvent e) {
				Point p = e.getPoint();
				int rowIndex = rowAtPoint(p);
				
				if (rowIndex < 0)
					return lastPoint;
				
				int colIndex = columnAtPoint(p);
				
				if (getRowCount() > 0 && (lastRow != rowIndex || lastCol != colIndex)) {
					lastPoint = new Point(p.x + 50, p.y - 300);
					lastRow = rowIndex;
					lastCol = colIndex;
				}
				
				return lastPoint;
				
			}
			
			
		};
		
		moleculeTable.addMouseListener(new MouseAdapter() {
			
						
			@Override
			public void mouseExited(MouseEvent arg0) {
				ToolTipManager.sharedInstance().setEnabled(true);
				
			}
				
			
		});
	
		moleculeTable.setAutoCreateColumnsFromModel(true);
		moleculeTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		
		//Make a sorter for the Table
		//moleculeTable.setAutoCreateRowSorter(true);
		TableRowSorter<TableModel> sorter = new TableRowSorter<TableModel>(tableModel);
		//System.err.println("Column Class: " + tableModel.getColumnClass(2).getSimpleName());
		
		moleculeTable.setRowSorter(sorter);
		
		
		moleculeTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
			
			@Override
			public void valueChanged(ListSelectionEvent e) {
				// TODO Auto-generated method stub
				tableSelectionChange();
			}
		});
		
		
		
		
		//Add Table to ScrollPane
		moleculeTable.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
		tableScrollPane = new JScrollPane(moleculeTable, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		moleculeTable.setFillsViewportHeight(true);
		
		//Add Scroll Pane to Panel
		tablePanel.add(tableScrollPane, BorderLayout.CENTER);
		
		//Add Chart Panel
		//tablePanel.add(chartPanel, BorderLayout.NORTH);
		
		//Add a button to export current table and crop selection
		JPanel buttonPanel = new JPanel(new GridLayout(2,3));
		
		
		buttonPanel.add(bShowMolList);
		buttonPanel.add(bCalcPath);
		buttonPanel.add(bPathBuilder);
		
		buttonPanel.add(bExportCSV);
		buttonPanel.add(bLeaveSelection);
		buttonPanel.add(bSavePNG);
		
		//Add an action listener that crops the selection
		bLeaveSelection.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				dropSelection();
				
			}
			
		});
		
		//Add Listener to export cvs
		bExportCSV.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				
				if (!exportCSV())
					System.err.println("Error writing csv!");
				else
					System.err.println("CSV written!");
				
			}
			
		});
		
		//Add Listener to showMolecule
		bSavePNG.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				// TODO Code for saving PNG here
				if (!saveAsPNG())
					System.err.println("Error occurred while png was trying to be saved!");
				
				
			}
			
		});
		
		
		//Add button Panel to table Panel
		tablePanel.add(buttonPanel, BorderLayout.SOUTH);
				
		
	}
	
	//CVS exporter
	private boolean exportCSV() {
		
		if (moleculeTable.getRowCount() == 0)
			return false;
		
		int returnVal = saveCSVChooser.showSaveDialog(this);
		
		if (returnVal != JFileChooser.APPROVE_OPTION) 
			return false;
		
		
		File file = saveCSVChooser.getSelectedFile();
		
		
		//Prepare the list to write
		List<String[]> tablelist = new ArrayList<String[]>();
		
		String[] header = new String[moleculeTable.getColumnCount() + 1]; //+1 for the SMILES
		header[0] = "SMILES";
		
		//Write out column names
		for (int i = 0; i < moleculeTable.getColumnCount(); i++) {
			
			header[i+1] = moleculeTable.getColumnName(i);
			
		}
		tablelist.add(header);
		
				

		for (int r = 0; r < tableRows.size(); r++) {
			String[] curRow = new String[moleculeTable.getColumnCount() + 1];
			
			curRow[0] = tableRows.get(r).getMol().getSmiles();
			
			//For the rest, get the cells from the model
			for (int c = 0; c < moleculeTable.getColumnCount(); c++) {
				
				int vc = c;
				c = moleculeTable.convertColumnIndexToModel(c);
				
				TableEntry te = (TableEntry) moleculeTable.getModel().getValueAt(r, c);
				curRow[vc+1] = te.toCVSString();
				
			}
			
			System.err.println(curRow);
			
			tablelist.add(curRow);
			
			
		}
		
		
		
		
		try {
				CSVWriter csvWriter = new CSVWriter(new FileWriter(file), ',', CSVWriter.NO_QUOTE_CHARACTER);
				
				csvWriter.writeAll(tablelist);
				
				csvWriter.close();
				
		} catch (IOException e) {
				
			return false;
		}
		
		
		
		return true;
	}
	
	
	private void dropSelection () {
		
		int row = moleculeTable.getSelectedRow();
		
		NSG.NSGVertex[] vs = new NSG.NSGVertex[row+1];
		
		
		for (int i = 0; i <= row; i++){
			
			vs[i] = tableRows.get(moleculeTable.convertRowIndexToModel(i));
					
		}
		
		pickItemListener.switchOff();
		
		
		vv1.getPickedEdgeState().clear();
		vv1.getPickedVertexState().clear();
		
		
		for (NSG.NSGVertex v : vs) {
			
			vv1.getPickedVertexState().pick(v, true);
			
		}
		
		pickItemListener.switchOn();
		
	}
	
	
	private void initSdPanel () {
		//Set up sdPanel
       // sdPanel.setPreferredSize(preferredSize2);
	}
	
	private void initLowerPanel() {
		
		
		
		initTablePanel();
		lowerPanel.add(tablePanel);
		
		//Add SD Panel
		initSdPanel ();
		//lowerPanel.add(sdPanel);
		
		lowerPanel.setPreferredSize(new Dimension(900,300));
		
		
	}
	
	
	protected boolean isVertexSelectedInTable(NSG.NSGVertex v) {
		
		if (moleculeTable.getSelectedRow() == -1)
			return false;
		
		int curRow = moleculeTable.convertRowIndexToModel(moleculeTable.getSelectedRow());
		
		if (curRow == -1)
			return false;
		
		if (tableRows.get(curRow).equals(v))
			return true;
		else
			return false;
		
	}
	
	private void initNSGPanel () {
		
		//Add Viewers to nsgPanel and rightPanel
        GraphZoomScrollPane gzsp = new GraphZoomScrollPane(vv1);
        nsgPanel.add(gzsp);
		initRightPanel();
        nsgPanel.add(rightPanel, BorderLayout.EAST);
        initLowerPanel();
        nsgPanel.add(lowerPanel, BorderLayout.SOUTH);
        
       
	}
	
	
	private void setColumnNames() {
		
		HashSet<String> allDNames = new HashSet<String>();
		
		for (Molecule m : nsg.allMolecules())
		{
			allDNames.addAll(m.getDescriptorNames());
		}
		
		
		ArrayList<String> allDNamesList = new ArrayList<String>();
		
		
		
	
		allDNamesList.add("Id");
		allDNamesList.add(nsg.getCentralPropName());
		allDNamesList.add("Discontinuity");
		allDNamesList.add("Tc");
		allDNamesList.add("Cliff Index");
		allDNamesList.add("Marker Sets");
		allDNamesList.addAll(allDNames);
		
		tableColumnNames = allDNamesList.toArray(new String[allDNames.size()] );
		
	}
	
	
	
	private NSG.NSGVertex getVertexBasedOnExternalId (String extId) {
		
		for (NSG.NSGVertex v : nsGraph.getVertices()) {
			if (v.getMol().getExternalId().equals(extId))
				return v;
				
		}
		return null;
	}
	
	
	/**
	 * 
	 * Selects one(!) Molecule based on its external id.
	 * Clears the current selection state.
	 * If you can, use selectBasedOnMoleculeSet instead.
	 * 
	 * @param extId External Id of the Molecule to be selected
	 */
	public void selectBasedOnExternalId(String extId) {
		
		NSG.NSGVertex v = getVertexBasedOnExternalId (extId);
		
		if (v != null) {
			
			//pickItemListener.switchOff();
			
			//vv1.getPickedVertexState().clear();
			//vv1.getPickedEdgeState().clear();
			
			
			
			
			vv1.getPickedVertexState().pick(v, true);
			//pickItemListener.switchOn();
		}
		else
			System.err.println("Graph does not contain Vertex with extId " + extId);
		
	}
	
	
	/**
	 * 
	 * Selects Molecules based on an external id collection.
	 * Clears the current selection state.
	 * If you can, use selectBasedOnMoleculeSet instead.
	 * 
	 * @param extIds Collection of external Ids to be picked
	 */
	public void selectBasedOnExternalIds(Collection<String> extIds) {
		
		
		pickItemListener.switchOff();
		vv1.getPickedVertexState().clear();
		
		
		for (String extId : extIds) {
		
			NSG.NSGVertex v = getVertexBasedOnExternalId (extId);
			
			if (v != null) {
				vv1.getPickedVertexState().pick(v, true);
			}
			else
				System.err.println("Graph does not contain Vertex with extId " + extId);
		}
		
		pickItemListener.switchOn();
		
	}
	
	/**
	 * 
	 * @return Collection of Molecule Objects that correspond to the picked Vertices
	 */
	public Collection<Molecule> getPickedMolecules() {
		
		Collection<Molecule> mc = new HashSet<Molecule>();
		
		for (NSG.NSGVertex v : vv1.getPickedVertexState().getPicked()) {
			mc.add(v.getMol());
		}
		
		return mc;
		
	}
	
	
	/**
	 * 
	 * Clears the current selection and picks all molecules that
	 * are present in the provided set.
	 * If a molecule from the set is not present in the NSG, no error is thrown.
	 * 
	 * @param ms Molecules to be picked
	 */
	public void selectBasedOnMoleculeSet (Collection<Molecule> ms) {
		
		vv1.getPickedVertexState().clear();
		vv1.getPickedEdgeState().clear();
		for (NSG.NSGVertex v : nsGraph.getVertices()) {
			if (ms.contains(v.getMol()))
				vv1.getPickedVertexState().pick(v, true);
		}
		
	}
	
	
	/**
	 * Function to add a Listener from a parent component
	 * that listens to the pick state of the NSG.
	 * 
	 * @param il Listener that listens to the picked state
	 */
	public void addPickedStateListener (ItemListener il) {
		vv1.getPickedVertexState().addItemListener(il);
	}
	
	
	public void addSynchroButtonListener (ActionListener al) {
		bSynchro.addActionListener(al);
	}
	
	/*
	public void addSARButtonListener (ActionListener al) {
		bAllSAR.addActionListener(al);
	}
	
	public void addSSRButtonListener (ActionListener al) {
		bAllSSR.addActionListener(al);
	}
	*/
	
	public void addMoleculeDepictionMouseListener (MouseListener ml) {
		sdPanel.addMouseListener(ml);
	}
	
	public Molecule getCurrentlyDepictedMol () {
		return sdPanel.getDepictedMol();
	}
	
	
	
	
	private void addListeners() {
		
		
		bShowMolList.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				molTableFrame.setVisible(true);
				molTableFrame.toFront();
				try {
					molTableFrame.setSelected(true);
				} catch (PropertyVetoException e1) {
					
				}
				parentPane.getDesktopManager().activateFrame(molTableFrame);
			
			}
		});
		
		bPathBuilder.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				if (!makePB())
					JOptionPane.showMessageDialog(parentPane, "Selected molecules do not form a valid path!", "No Path!", JOptionPane.ERROR_MESSAGE);
			
			}
		});

		
		bCalcPath.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				if (!pickPath())	
					JOptionPane.showMessageDialog(parentPane, "No Path found!", "No Path!", JOptionPane.ERROR_MESSAGE);
			}
		});
		
		
		
		
		this.addInternalFrameListener(new InternalFrameAdapter() {
			
			
			@Override
			public void internalFrameClosed(InternalFrameEvent e) {
				molTableFrame.dispose();
				nsg.getSarSet().removeAllCDKMols();
				
				//dispose of all the listeners we added to the menus
				NSGMouseMenuManager.sharedInstance().menuItemSarTree.removeActionListener(mmListenerSarTree);
				NSGMouseMenuManager.sharedInstance().menuItemSarTree.removeActionListener(mmListenerSelectNeighbors);
				
			}
			
			@Override
			public void internalFrameActivated(InternalFrameEvent e) {
				

				NSGMouseMenuManager.sharedInstance().setActiveNSGFrame((NSGFrame) e.getInternalFrame());
				NSGMouseMenuManager.sharedInstance().setState(NSGMouseMenuManager.State.NSG);
				sharedPickedState.setAllPickable();
				
				
			}
			
			
		});
		
		NSGMouseMenuManager.sharedInstance().menuItemSarTree.addActionListener(mmListenerSarTree);
		NSGMouseMenuManager.sharedInstance().menuItemSelectNeighbors.addActionListener(mmListenerSelectNeighbors);
	}
	
	public NSGFrame(NSG graph, JDesktopPane parentpane, boolean showEdges) {
		
		super();
		
		super.setParentFrame(this);
		
		showEdgesOnInit = showEdges;
		nsg = graph;
		parentPane = parentpane;
		setColumnNames();
		
		
		nsGraph = graph.getGraph();
		//layout = new FRLayout2<NSG.NSGVertex, NSG.NSGEdge>(nsGraph);
		//layout = new ISOMLayout<NSG.NSGVertex, NSG.NSGEdge>(nsGraph);
		layout = new MyFRLayout(nsGraph, preferredSize1);
		//((FRLayout2<NSG.NSGVertex, NSG.NSGEdge>) layout).setMaxIterations(nsGraph.getVertexCount() * LayoutConstants.graphLayoutMaxIterationsMultiplier);
		initgraph();
		
		Container content = this.getContentPane();
	    
		initNSGPanel();
		content.add(nsgPanel);
		
				
		molTableFrame = new MoleculeTableFrame(moleculeTable);
		parentPane.add(molTableFrame);
		
		addListeners();		
		
		
		this.setSize(new Dimension(800,800));
        this.setTitle(nsg.graphTitle());
        this.setClosable(true);
        this.setIconifiable(true);
        this.setDefaultCloseOperation(JInternalFrame.DISPOSE_ON_CLOSE);
        try {
			this.setIcon(true);
		} catch (PropertyVetoException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		this.pack();
		this.toFront();
		this.repaint();
        
        
		
	}
	
	
	 
	
	

}
