/*
 * This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

package bit.lsi.sar.nsg.swing;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.Paint;
import java.awt.Stroke;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.ConcurrentModificationException;

import javax.swing.JButton;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JToolTip;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

import org.apache.commons.collections15.Transformer;

import bit.lsi.sar.nsg.NSG;
import bit.lsi.sar.nsg.NSG.NSGVertex;
import bit.lsi.sar.nsg.mousemenus.NSGMouseMenuManager;
import bit.lsi.sar.nsg.pathbuilder.PBLayout;
import bit.lsi.sar.nsg.pathbuilder.PathBuilder;
import bit.lsi.sar.nsg.pathbuilder.transformers.LayoutConstants;
import bit.lsi.sar.nsg.pathbuilder.transformers.MutablePickedState;
import edu.uci.ics.jung.algorithms.layout.GraphElementAccessor;
import edu.uci.ics.jung.algorithms.layout.Layout;
import edu.uci.ics.jung.visualization.GraphZoomScrollPane;
import edu.uci.ics.jung.visualization.Layer;
import edu.uci.ics.jung.visualization.RenderContext;
import edu.uci.ics.jung.visualization.VisualizationViewer;
import edu.uci.ics.jung.visualization.VisualizationServer.Paintable;
import edu.uci.ics.jung.visualization.control.CrossoverScalingControl;
import edu.uci.ics.jung.visualization.control.GraphMouseListener;
import edu.uci.ics.jung.visualization.control.PickingGraphMousePlugin;
import edu.uci.ics.jung.visualization.control.PluggableGraphMouse;
import edu.uci.ics.jung.visualization.control.ScalingControl;
import edu.uci.ics.jung.visualization.control.ScalingGraphMousePlugin;
import edu.uci.ics.jung.visualization.control.TranslatingGraphMousePlugin;
import edu.uci.ics.jung.visualization.decorators.EdgeShape;
import edu.uci.ics.jung.visualization.renderers.BasicRenderer;
import edu.uci.ics.jung.visualization.renderers.Renderer;
import edu.uci.ics.jung.visualization.transform.MutableTransformer;

public class PathBuilderFrame extends AbstractNSGFrame {

	
	
	class SensiblePaintTransformer<I> implements Transformer<I,Paint> {

		private Transformer<I,Paint> orgt;
		
		public SensiblePaintTransformer(Transformer<I,Paint> orgt) {
			
			this.orgt = orgt;
			
		}
		
		
		@Override
		public Paint transform(I v) {
			Paint originalPaint = orgt.transform(v);
			
			switch (curGraphState) {
			
			case PATH:
				return originalPaint;
			
			case RADIAL:
			case LIST:
				
				
			if (v.getClass().equals(NSG.NSGVertex.class) && !pb.isPath((NSG.NSGVertex) v))
				return originalPaint;
			
			Color col = (Color) originalPaint;
			float[] hsbvals = new float[3];
			Color.RGBtoHSB(col.getRed(), col.getGreen(), col.getBlue(), hsbvals);
			
			if (v.getClass().equals(NSG.NSGEdge.class))
				return Color.getHSBColor(hsbvals[0], hsbvals[1], 1-LayoutConstants.selectionBleakness);
			
			//in Path AND in Radial / List
			if (v.getClass().equals(NSG.NSGVertex.class) && ( pb.isRadial((NSG.NSGVertex)v) || pb.isList((NSG.NSGVertex)v)))
				return Color.getHSBColor(hsbvals[0], LayoutConstants.selectionHalfBleakness, hsbvals[2]);
			
			return Color.getHSBColor(hsbvals[0], LayoutConstants.selectionBleakness, hsbvals[2]);
			
			default:
				return originalPaint;
			}
			
			
		}
		
	}
	
	
	private static final long serialVersionUID = 1L;
	
	
	private VisualizationViewer<NSG.NSGVertex, NSG.NSGEdge> vv;
	private VisualizationViewer<NSG.NSGVertex, NSG.NSGEdge> parentVv;
	
	
	//Control Panel and Buttons
	private JPanel panelControls = new JPanel();
	
	private JButton btShowList = new JButton("List");
	private JButton btShortcut = new JButton("Shortcut");
	private JLabel lTc = new JLabel();
	
	
	
	private PBLayout layout;
	
	private PathBuilder pb;
	
	class MyItemListener implements ItemListener {

		@Override
		public void itemStateChanged(ItemEvent e) {
			 itemStateChanged();
			
		}
		
		public void itemStateChanged() {
			
			switch (curGraphState) {
			
			case PATH:
				
				btShowList.setEnabled(listPossible());
				btShortcut.setEnabled(shortcutPossible());
				
				break;
				
			default:
				btShortcut.setEnabled(false);
				btShowList.setEnabled(false);
			}
			
			//btShortcut.revalidate();
			//btShowList.revalidate();
			setTcLabel();
			
		}
		
	}
	
	
	private MyItemListener pickChangeListener = new MyItemListener();
	
	private enum GraphState {PATH, RADIAL, LIST};
	private GraphState curGraphState = GraphState.PATH;
	
	
	//Mouse Menu Listeners
	private ActionListener mmListenerRadial = new ActionListener() {

		@Override
		public void actionPerformed(ActionEvent e) {
			radialView(NSGMouseMenuManager.sharedInstance().menuItemRadial.getVertex());			
		}
		
	};
	
	
	
	
	private void makeColorful() {
		
		
		
		Renderer<NSG.NSGVertex, NSG.NSGEdge> renderer = new BasicRenderer<NSG.NSGVertex, NSG.NSGEdge> () {
			
			@Override
			public void render(RenderContext<NSG.NSGVertex, NSG.NSGEdge> renderContext, Layout<NSG.NSGVertex, NSG.NSGEdge> layout) {
				
				// paint all the edges
		        try {
		        	for(NSG.NSGEdge e : layout.getGraph().getEdges()) {

				        renderEdge(
				                renderContext,
				                layout,
				                e);
				        renderEdgeLabel(
				                renderContext,
				                layout,
				                e);
		        	}
		        } catch(ConcurrentModificationException cme) {
		        	renderContext.getScreenDevice().repaint();
		        }
				
				// paint all the vertices
		        
		        
		        ArrayList<NSG.NSGVertex> prioritizedVertices = new ArrayList<NSG.NSGVertex>();
		        
		        prioritizedVertices.addAll(pb.getPathInGraph());
		        prioritizedVertices.addAll(pb.getListInGraph());
		        prioritizedVertices.addAll(pb.getRadialInGraph());
		        
		        
		        try {
		        	for(NSG.NSGVertex v : prioritizedVertices) {

				    	renderVertex(
				                renderContext,
		                        layout,
				                v);
				    	renderVertexLabel(
				                renderContext,
		                        layout,
				                v);
		        	}
		        } catch(ConcurrentModificationException cme) {
		            renderContext.getScreenDevice().repaint();
		        }
			}
			
			
		};
		
		
		
		final RenderContext<NSG.NSGVertex,NSG.NSGEdge> prc = parentVv.getRenderContext();
		final RenderContext<NSG.NSGVertex,NSG.NSGEdge> rc = vv.getRenderContext();
		
			
				
		rc.setVertexDrawPaintTransformer(new SensiblePaintTransformer<NSG.NSGVertex>(prc.getVertexDrawPaintTransformer()));
		rc.setVertexFillPaintTransformer(new SensiblePaintTransformer<NSG.NSGVertex>(prc.getVertexFillPaintTransformer()));
		
		rc.setEdgeArrowStrokeTransformer(prc.getEdgeArrowStrokeTransformer());
		rc.setVertexStrokeTransformer( new Transformer<NSG.NSGVertex, Stroke>() {

			@Override
			public Stroke transform(NSGVertex v) {
				BasicStroke originalStroke = (BasicStroke) prc.getVertexStrokeTransformer().transform(v);
				
				if (v.hasAddNeighbor())
					return new BasicStroke(originalStroke.getLineWidth() + 5);
				else
					return originalStroke;
					
				
			}
			
		});
				
				
		
		rc.setEdgeArrowPredicate(new RenderContext.UndirectedEdgeArrowPredicate<NSG.NSGVertex,NSG.NSGEdge>());
		
		rc.setEdgeShapeTransformer(new EdgeShape.Line<NSG.NSGVertex, NSG.NSGEdge>());
		rc.setEdgeDrawPaintTransformer(new SensiblePaintTransformer<NSG.NSGEdge>(prc.getEdgeDrawPaintTransformer()));
		
		//vv.getRenderer().setEdgeRenderer(new SensibleEdgeRenderer());
		//vv.getRenderer().setVertexRenderer(new SensibleVertexRenderer());
		
		
		vv.setPickedVertexState(prc.getPickedVertexState());
		vv.setPickedEdgeState(prc.getPickedEdgeState());
		
		vv.setVertexToolTipTransformer(new ToolTipShowMol());
		vv.setEdgeToolTipTransformer(new ToolTipForEdge());
		
		vv.setRenderer(renderer);
	
		//ToolTipManager.sharedInstance().setInitialDelay(10);		
	}
	
	
	
	private boolean shortcutPossible()  {
		
		ArrayList<NSG.NSGVertex> pickedHere = new ArrayList<NSG.NSGVertex>();
		pickedHere.addAll(vv.getPickedVertexState().getPicked());
		pickedHere.retainAll(pb.getPath());
		
		pb.sortVertices(pickedHere);
		
		
		if (pickedHere.size() != 2)
			return false;
		
		if (pb.getGraph().findEdge(pickedHere.get(0), pickedHere.get(1)) != null)
			return false;
				
		if (pb.getNSG().getGraph().findEdge(pickedHere.get(0), pickedHere.get(1)) != null)
			return true;
		
		return false;
	}
	
	
	private void setTcLabel() {
		ArrayList<NSG.NSGVertex> pickedHere = new ArrayList<NSG.NSGVertex>();
		pickedHere.addAll(vv.getPickedVertexState().getPicked());
		pickedHere.retainAll(pb.getPath());
		
		if (pickedHere.size() == 2)
			lTc.setText("Tc: " + String.format("%.2f", pb.getNSG().getTc(pickedHere.get(0), pickedHere.get(1))));
		else
			lTc.setText("");
			
		
		
	}
	
	private boolean listPossible() {
		ArrayList<NSG.NSGVertex> pickedHere = new ArrayList<NSG.NSGVertex>();
		
		
		
		pickedHere.addAll(vv.getPickedVertexState().getPicked());
		pickedHere.retainAll(pb.getPath());
		
		pb.sortVertices(pickedHere);
		
		if (pickedHere.size() > 2 || pickedHere.size() == 0)
			return false;
		
		
		//are we at the end of a path?
		if (pickedHere.size() == 1 &&
		(pickedHere.get(0).equals(pb.getPath().get(0)) ||
		pickedHere.get(0).equals(pb.getPath().get(pb.getPath().size() -1) ) ))
			return false;
		
		
		if (pickedHere.size() == 2)		
			return pb.listPossible(pickedHere.get(0), pickedHere.get(1));
		else 
			return pb.listPossible(pickedHere.get(0), pickedHere.get(0));
				
	}
	
	
	private boolean showList() {
		
		ArrayList<NSG.NSGVertex> pickedHere = new ArrayList<NSG.NSGVertex>();
		pickedHere.addAll(vv.getPickedVertexState().getPicked());
		pickedHere.retainAll(pb.getPath());
		
		pb.sortVertices(pickedHere);
		
		
		if (pickedHere.size() > 2 || pickedHere.size() == 0)
			return false;
		
		
		//are we at the end of a path?
		if (pickedHere.size() == 1 &&
		(pickedHere.get(0).equals(pb.getPath().get(0)) ||
		pickedHere.get(0).equals(pb.getPath().get(pb.getPath().size() -1) ) ))
			return false;
		
		
		
		if (pickedHere.size() == 2) {
			
			if (pb.showList(pickedHere.get(0), pickedHere.get(1))){
				
				setGraphState(GraphState.LIST);
			
				vv.repaint();
			
				return true;
				
			}
			else
				return false;
		
		}
		else {
			if (pb.showList(pickedHere.get(0), pickedHere.get(0))){
				
				setGraphState(GraphState.LIST);
			
				vv.repaint();
			
				return true;
				
			}
			else
				return false;
		}
		
	}
	
	private void radialView(NSG.NSGVertex v) {
		vv.addPreRenderPaintable(pb.getRings());
		pb.showRadialNeighbors(v);
		setGraphState(GraphState.RADIAL);
	}
	
	
	private void setGraphState(GraphState s) {
		
		curGraphState = s;
		
		MutablePickedState mps = (MutablePickedState) vv.getPickedVertexState();
		
		switch (curGraphState) {
		
		case PATH:
			NSGMouseMenuManager.sharedInstance().menuItemRadial.setEnabled(true);
			mps.setAllPickable();
			break;
		
		case LIST:
			
			NSGMouseMenuManager.sharedInstance().menuItemRadial.setEnabled(false);

			for (NSG.NSGVertex v: pb.getPathInGraph())
				mps.setPickable(v, pb.isList(v));
			
			
			break;
			
			
		case RADIAL:
			NSGMouseMenuManager.sharedInstance().menuItemRadial.setEnabled(false);
			

			for (NSG.NSGVertex v: pb.getPathInGraph())
				mps.setPickable(v, pb.isRadial(v));
			
			break;
		
		}
		layout.reset();
		vv.repaint();
	}
	
	private void initVV() {
		
		vv.setBackground(Color.white);
		//vv.setRenderContext(parentVv.getRenderContext());
		
		vv.getRenderContext().getMultiLayerTransformer().setToIdentity();
		
		vv.addPreRenderPaintable(pb.getAxes());
		
		
		Point2D upperLeft = pb.getUpperLeftPoint();
		
		ScalingControl scaler = new CrossoverScalingControl();
		
		
		vv.getRenderContext().getMultiLayerTransformer().getTransformer(Layer.LAYOUT).translate(-upperLeft.getX(), -upperLeft.getY());
		
		upperLeft = vv.getRenderContext().getMultiLayerTransformer().getTransformer(Layer.LAYOUT).transform(upperLeft);
		
		scaler.scale(vv, (float)( (pb.getWidth() /vv.getSize().getWidth())), upperLeft);
		 
         
		
		
		// create a GraphMouse for the main view
        PluggableGraphMouse graphMouse = new PluggableGraphMouse();
        
        //Make the wheel do the navigating
        graphMouse.add(new TranslatingGraphMousePlugin(MouseEvent.BUTTON2_MASK));
        graphMouse.add(new ScalingGraphMousePlugin(new CrossoverScalingControl(), 0, 1.1f, 0.9f));
        
        //The left button is used to pick things. If shift is pressed then we add to the selection
        graphMouse.add(new PickingGraphMousePlugin<NSG.NSGVertex,NSG.NSGEdge>(MouseEvent.BUTTON1_MASK, MouseEvent.BUTTON1_MASK | MouseEvent.SHIFT_MASK));
        graphMouse.add(NSGMouseMenuManager.sharedInstance().getMousePlugin());
        
        //Add the mouse to the main view
        vv.setGraphMouse(graphMouse);
        
        makeColorful();
        
        
        vv.addMouseListener(new MouseAdapter() {
			
			@Override
			public void mouseClicked(MouseEvent e) {
				Point2D p = e.getPoint();
				
				GraphElementAccessor<NSG.NSGVertex,NSG.NSGEdge> pickSupport = vv.getPickSupport();
				
				if(pickSupport == null || pickSupport.getVertex(vv.getGraphLayout(), p.getX(), p.getY()) == null) {
        			
					switch (curGraphState) {
					
					case PATH:
						vv.getPickedVertexState().clear();
						break;
					case LIST:
						pb.abortList();
						setGraphState(GraphState.PATH);
						break;
					case RADIAL:
						pb.abortRadial();
						vv.removePreRenderPaintable(pb.getRings());
						setGraphState(GraphState.PATH);
						break;
					}
					
				}
				
				layout.reset();
				vv.repaint();
        	
        		}
					
				
			
		});
        
        vv.addKeyListener(new KeyListener() {
			
			@Override
			public void keyTyped(KeyEvent e) {
				if (e.getKeyChar() == KeyEvent.VK_L ) {
					
					showList();
					
				}
				else if (e.getKeyChar() == KeyEvent.VK_ESCAPE) {
					
					switch (curGraphState) {
					
					case PATH:
						vv.getPickedVertexState().clear();
						break;
					case LIST:
						pb.abortList();
						setGraphState(GraphState.PATH);
						break;
					case RADIAL:
						pb.abortRadial();
						vv.removePreRenderPaintable(pb.getRings());
						setGraphState(GraphState.PATH);
						break;
					}
					
				}
				
				layout.reset();
				vv.repaint();
				
			}
			
			@Override
			public void keyReleased(KeyEvent e) {
				
			}
			
			@Override
			public void keyPressed(KeyEvent e) {
				
				
			}
		});
        
        
        vv.addGraphMouseListener(new GraphMouseListener<NSG.NSGVertex>() {

			@Override
			public void graphClicked(NSGVertex v, MouseEvent e) {
				
				
				
				
				switch (curGraphState) {
				
				case PATH: 
					
					if (e.getButton() == MouseEvent.BUTTON1 && e.getClickCount() == 2)
						radialView(v);
					
					break;
				
				case RADIAL:
					
					if (e.getButton() == MouseEvent.BUTTON1 && !pb.isCentral(v) && pb.isRadial(v)) 
						pb.addRadialVertexToPath(v);
					else
						pb.abortRadial();
						
					vv.removePreRenderPaintable(pb.getRings());
					setGraphState(GraphState.PATH);
					
					break;
					
				case LIST:
					
					if (e.getButton() == MouseEvent.BUTTON1 && pb.isList(v)) 
						pb.addVertexToPath(v);
					else
						pb.abortList();
					
					setGraphState(GraphState.PATH);
										
				}
				
				layout.reset();
				vv.repaint();
			
			}
				
			

			@Override
			public void graphPressed(NSGVertex arg0, MouseEvent arg1) {
				
				
			}

			@Override
			public void graphReleased(NSGVertex arg0, MouseEvent arg1) {
				
				
			}
        	
        });
        
       
        vv.getPickedVertexState().addItemListener(pickChangeListener);
        
        btShortcut.setEnabled(false);
		btShowList.setEnabled(false);
        
       
		
	}
	
	
	private void shortCut() {
		
		ArrayList<NSG.NSGVertex> pickedHere = new ArrayList<NSG.NSGVertex>();
		pickedHere.addAll(vv.getPickedVertexState().getPicked());
		pickedHere.retainAll(pb.getPath());
		
		pb.shortcut(pickedHere.get(0), pickedHere.get(1));
		
		layout.reset();
		vv.repaint();
		
	}
	
	private void initControls() {
		
		
		panelControls.setLayout(new GridLayout(3,1));
		
		panelControls.add(lTc);
		panelControls.add(btShowList);
		panelControls.add(btShortcut);
		
		
		btShowList.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				showList();
				pickChangeListener.itemStateChanged();
				btShowList.repaint();
						
			}		
		});
		
		btShortcut.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent e) {
				shortCut();
				pickChangeListener.itemStateChanged();
				btShortcut.repaint();
			}
		});
		
		
		this.getContentPane().add(panelControls);
		
	}
	
	
	private void addListeners() {
		
		this.addInternalFrameListener(new InternalFrameAdapter() {
			
			@Override
			public void internalFrameActivated(InternalFrameEvent arg0) {
				NSGMouseMenuManager.sharedInstance().setState(NSGMouseMenuManager.State.PATHBUILDER);
				NSGMouseMenuManager.sharedInstance().setActiveNSGFrame(getParentFrame());
				
				
				
			}
			
			@Override
			public void internalFrameClosed(InternalFrameEvent e) {
				
				NSGMouseMenuManager.sharedInstance().menuItemRadial.removeActionListener(mmListenerRadial);
				
				
			}
		});
		
		NSGMouseMenuManager.sharedInstance().menuItemRadial.addActionListener(mmListenerRadial);
		
		
	}
	
	
	public PathBuilderFrame(VisualizationViewer<NSG.NSGVertex, NSG.NSGEdge> parent, NSG nsg, Collection<NSG.NSGVertex> vs, final NSGFrame parentFrame) {
		super("Path Builder", parentFrame);
		
		parentVv = parent;
		pb = new PathBuilder(nsg, vs);
		layout = new PBLayout(pb);
		
		this.molToolTip = new MoleculeToolTip(sdPanel, nsg);
				
		vv = new VisualizationViewer<NSG.NSGVertex, NSG.NSGEdge>(layout) {

			/**
			 * 
			 */
			private static final long serialVersionUID = 1L;
			
			@Override
			public JToolTip createToolTip() {
				
				if (overVertex)
					return molToolTip;
				else
					return new JToolTip();
				
			}
			
			
			
		};
		pb.setVisualizationViewer(vv);
		
		
		vv.setPreferredSize(new Dimension(1800, 400));
		initVV();
		
		GraphZoomScrollPane gzsp = new GraphZoomScrollPane(vv);
		
		this.setLayout(new FlowLayout());
		
		gzsp.setPreferredSize(new Dimension(900, 400));
		this.getContentPane().add(gzsp);
		//this.getContentPane().add(sdPanel);
		
		
		initControls();
		addListeners();
		
		this.setClosable(true);
		this.setDefaultCloseOperation(JInternalFrame.DISPOSE_ON_CLOSE);
		this.pack();
		this.show();
		
					
		
	}
	
	
	

}
