/*
 * This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */


/**
 * 
 */
package bit.lsi.sar.nsg.swing;


import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Paint;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.util.Collection;

import javax.swing.JInternalFrame;
import javax.swing.JToolTip;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

import org.apache.commons.collections15.Predicate;
import org.apache.commons.collections15.Transformer;

import bit.lsi.sar.nsg.NSG;
import bit.lsi.sar.nsg.SARTree;
import bit.lsi.sar.nsg.NSG.NSGEdge;
import bit.lsi.sar.nsg.NSG.NSGVertex;
import bit.lsi.sar.nsg.mousemenus.NSGMouseMenuManager;
import edu.uci.ics.jung.algorithms.layout.Layout;
import edu.uci.ics.jung.algorithms.layout.TreeLayout;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.util.Context;
import edu.uci.ics.jung.visualization.DefaultVisualizationModel;
import edu.uci.ics.jung.visualization.RenderContext;
import edu.uci.ics.jung.visualization.VisualizationModel;
import edu.uci.ics.jung.visualization.VisualizationViewer;
import edu.uci.ics.jung.visualization.control.CrossoverScalingControl;
import edu.uci.ics.jung.visualization.control.GraphMouseListener;
import edu.uci.ics.jung.visualization.control.PickingGraphMousePlugin;
import edu.uci.ics.jung.visualization.control.PluggableGraphMouse;
import edu.uci.ics.jung.visualization.control.ScalingGraphMousePlugin;
import edu.uci.ics.jung.visualization.control.TranslatingGraphMousePlugin;
import edu.uci.ics.jung.visualization.decorators.EdgeShape;
import edu.uci.ics.jung.visualization.util.VertexShapeFactory;

/**
 * 
 * Shows the SAR tree for a particular Node in the parent NSGFrame.
 * The Selection is synchronized.
 * 
 * @author lounkine
 *
 */
public class SARTreeFrame extends AbstractNSGFrame {

	/**
	 * 
	 */
	private static final long serialVersionUID = -3302093278069680643L;
	
	private class ToolTipForEdge implements Transformer<NSG.NSGEdge, String> {

		@Override
		public String transform(NSGEdge e) {
			
			overVertex = false;
			return sartree.getEdgePathWeightString(e);
		}
	
	}
	
	
	private SARTree sartree;
	private Graph<NSG.NSGVertex, NSG.NSGEdge> treeG;
	private VisualizationViewer<NSG.NSGVertex, NSG.NSGEdge> parentViewer;
	
	
	private VertexShapeFactory<NSG.NSGVertex> starFactory = new VertexShapeFactory<NSG.NSGVertex>(
			
			new Transformer<NSG.NSGVertex, Integer>() 
			{

				@Override
				public Integer transform(NSGVertex v) {
					// TODO Auto-generated method stub
					return (int) (20 + 40 * v.getSizeValue());
				}
				
			}, 
			
			new Transformer<NSG.NSGVertex, Float>() 
			{

				@Override
				public Float transform(NSGVertex v) {
					// TODO Auto-generated method stub
					return 1f;
				}
				
			});
	
	private NSG.NSGVertex centralV;
	
	private static Dimension preferredSize1 = new Dimension(400,800);
	
	private VisualizationModel<NSG.NSGVertex, NSG.NSGEdge> vm;
	private VisualizationViewer<NSG.NSGVertex, NSG.NSGEdge> vv1;
	
	private Layout<NSG.NSGVertex, NSG.NSGEdge> layout;
	
	
	
	//Mousemenu Listeners
	private ActionListener mmListenerShowInPB = new ActionListener() {
		
		@Override
		public void actionPerformed(ActionEvent e) {
			
			NSG.NSGVertex v = NSGMouseMenuManager.sharedInstance().menuItemShowPathInPB.getVertex();
			getParentFrame().makePB(sartree.getPathVertices(v));
			
		}
	};

	
	
	private void makeColorful() {
		
		
		vv1.setBackground(Color.white);
		
		
		Predicate<Context<Graph<NSG.NSGVertex, NSG.NSGEdge>, NSG.NSGEdge>> apredicate = new Predicate<Context<Graph<NSG.NSGVertex, NSG.NSGEdge>, NSG.NSGEdge>> () {

			@Override
			public boolean evaluate(
					Context<Graph<NSGVertex, NSGEdge>, NSGEdge> c) {
				// TODO Auto-generated method stub
				return true;
			}
			
		};
		
		
		
		//Set up a vertex stroke transformer
		Transformer<NSG.NSGVertex, Stroke> vertexStrokeTransformer = new Transformer<NSG.NSGVertex, Stroke> () {

			@Override
			public Stroke transform(NSG.NSGVertex v) {
				
					int addVal = 0;
				
					if(getParentFrame().isVertexSelectedInTable(v))
						addVal = 10;
						
				
				
				
					if (v.hasAddNeighbor()) 
						return new BasicStroke(5+addVal);
					else if (vv1.getPickedVertexState().isPicked(v))
						return new BasicStroke(3+addVal);
					else
						return new BasicStroke(0);
				
				
			}
			
			
			
		};
		
		
		
		Transformer<NSG.NSGVertex,Paint> vertexDrawPaintTransformer = new Transformer<NSG.NSGVertex,Paint>() {
			public Paint transform(NSG.NSGVertex v) {
					
				if (getParentFrame().isVertexSelectedInTable(v))
					return Color.magenta;
					
				
				else if (vv1.getPickedVertexState().isPicked(v))
					return Color.blue;
				else
					return v.getSetColor();
					
			}
			
		};
		
		
		
		RenderContext<NSG.NSGVertex, NSG.NSGEdge> prc = parentViewer.getRenderContext();
		RenderContext<NSG.NSGVertex, NSG.NSGEdge> rc = vv1.getRenderContext();
		
		rc.setEdgeArrowPredicate(apredicate);
		
		
		rc.setVertexFillPaintTransformer(prc.getVertexFillPaintTransformer());
		rc.setVertexDrawPaintTransformer(prc.getVertexDrawPaintTransformer());
		rc.setVertexStrokeTransformer(prc.getVertexStrokeTransformer());
		rc.setVertexShapeTransformer(prc.getVertexShapeTransformer());
		
		rc.setVertexStrokeTransformer(vertexStrokeTransformer);
		rc.setVertexDrawPaintTransformer(vertexDrawPaintTransformer);
		
        	
		
		//Set up an edge Transformer
		Transformer<NSG.NSGEdge, Paint> edgeColor = new Transformer<NSG.NSGEdge, Paint> () {
			public Paint transform(NSG.NSGEdge e) {
				
				
				
				
				float[] hsb = new float[3];
				
				boolean isP = vv1.getPickedEdgeState().isPicked(e);
								
				final double mv = 0.8;	//make visible
				
				
				if (isP) {
					Color.RGBtoHSB(0,0,255, hsb);
									
					hsb[1] = (float) 
							 sartree.getEdgePathWeight(e);
					
					
					hsb[2] = 1;
					hsb[1] = (float) (mv * hsb[1] + (1-mv));
				}
				else {
					hsb[0] = 0;
					hsb[2] = (float) (1 -   
							 sartree.getEdgePathWeight(e)
							);
					hsb[1] = 0;
					hsb[2] = (float) (mv * hsb[2] );
					
				}
				
				
				
				return Color.getHSBColor(hsb[0], hsb[1], hsb[2]);

			}
		};
		
		
		
		
		Transformer<NSG.NSGVertex, Shape> vertexShapeTransformer = new Transformer<NSG.NSGVertex, Shape>() {

			@Override
			public Shape transform(NSGVertex v) {
				// TODO Auto-generated method stub
				if (v.equals(centralV))
					return starFactory.getRegularStar(v, 5);
				else
					return parentViewer.getRenderContext().getVertexShapeTransformer().transform(v);
			}
			
		};
		
		
		vv1.getRenderContext().setEdgeDrawPaintTransformer(edgeColor);
		vv1.getRenderContext().setEdgeShapeTransformer(new EdgeShape.Line<NSG.NSGVertex, NSG.NSGEdge>());
		vv1.getRenderContext().setVertexShapeTransformer(vertexShapeTransformer);
		
		vv1.setVertexToolTipTransformer(new ToolTipShowMol());
		vv1.setEdgeToolTipTransformer(new ToolTipForEdge());
		
		
		// create a GraphMouse for the main view
        PluggableGraphMouse graphMouse = new PluggableGraphMouse();
        
        //Make the wheel do the navigating
        graphMouse.add(new TranslatingGraphMousePlugin(MouseEvent.BUTTON2_MASK));
        graphMouse.add(new ScalingGraphMousePlugin(new CrossoverScalingControl(), 0, 1.1f, 0.9f));
        
        //The left button is used to pick things. If shift is pressed then we add to the selection
        graphMouse.add(new PickingGraphMousePlugin<NSG.NSGVertex,NSG.NSGEdge>(MouseEvent.BUTTON1_MASK, MouseEvent.BUTTON1_MASK | MouseEvent.SHIFT_MASK));
        graphMouse.add(NSGMouseMenuManager.sharedInstance().getMousePlugin());
        
		
		vv1.setGraphMouse(graphMouse);
		
		vv1.setPickedVertexState(parentViewer.getPickedVertexState());
		
		
		vv1.addGraphMouseListener( new GraphMouseListener<NSG.NSGVertex>() {

			@Override
			public void graphClicked(NSGVertex v, MouseEvent me) {
				
			
				//System.err.println(me.getButton());
				
				if (me.getButton() == MouseEvent.BUTTON1){
					if (me.getClickCount() == 2) {
						
						Collection<NSG.NSGVertex> path = sartree.getPathVertices(v);
						
						//System.err.println(path);
						
						for (NSG.NSGVertex vv : treeG.getVertices())
						{
							vv1.getPickedVertexState().pick(vv, 
									(path.contains(vv) || vv1.getPickedVertexState().isPicked(vv))
							);
						}
						
						getParentFrame().updateTable();
						
					}
				}
							
			}

			@Override
			public void graphPressed(NSGVertex v, MouseEvent me) {
				
				
			}

			@Override
			public void graphReleased(NSGVertex v, MouseEvent me) {
				
				
			}
       	
        });
				
	}
	
	
	
	private void initgraph() {
		
		//int lDim = (int) Math.sqrt(treeG.getVertexCount()) * 30;
		
		layout = new TreeLayout<NSGVertex, NSGEdge>(sartree.getTree());
		//layout.setSize(new Dimension(lDim,lDim));
		
		
		
		
		vm = new DefaultVisualizationModel<NSG.NSGVertex, NSG.NSGEdge>(layout, preferredSize1);
		
		vv1 = new VisualizationViewer<NSG.NSGVertex, NSG.NSGEdge>(vm, preferredSize1) {

			/**
			 * 
			 */
			private static final long serialVersionUID = 1L;
			
			@Override
			public JToolTip createToolTip() {
				
				if (overVertex)
					return molToolTip;
				else
					return new JToolTip();
				
			}
				
		};
		
		
		//Set Transformers
        makeColorful ();
        
        
        //Set ItemListeners
        
        
        
		
		
	}
	
	
	private void addListeners() {
		
this.addInternalFrameListener(new InternalFrameAdapter() {
			
			
			@Override
			public void internalFrameClosed(InternalFrameEvent e) {
				
			//dispose of all the listeners we added to the menus
			NSGMouseMenuManager.sharedInstance().menuItemShowPathInPB.removeActionListener(mmListenerShowInPB);
				
			}
			
			@Override
			public void internalFrameActivated(InternalFrameEvent e) {
				

				NSGMouseMenuManager.sharedInstance().setActiveNSGFrame(getParentFrame());
				NSGMouseMenuManager.sharedInstance().setState(NSGMouseMenuManager.State.SARTREE);
				
				
			}
			
			
		});
		
		NSGMouseMenuManager.sharedInstance().menuItemShowPathInPB.addActionListener(mmListenerShowInPB);
	
		
		
	}
	
	
	public SARTreeFrame (NSGFrame parentFrame, VisualizationViewer<NSG.NSGVertex, NSG.NSGEdge> parent, NSG nsg, NSG.NSGVertex startVertex) {
		
		super(startVertex.getMol().toString(), parentFrame);
		sartree = new SARTree(nsg, startVertex);
		treeG = sartree.getGraph();
		parentViewer = parent;
		centralV =  startVertex;
		
		
		this.molToolTip = new MoleculeToolTip(sdPanel, nsg);
		initgraph();
		
		
		this.getContentPane().add(vv1);
		
		addListeners();
		this.setClosable(true);
		this.pack();
		this.show();
		this.toFront();
		this.setDefaultCloseOperation(JInternalFrame.DISPOSE_ON_CLOSE);
		
	}
	
	
	public void addGraphPickItemListener (ItemListener il) {
		
		vv1.getPickedVertexState().addItemListener(il);
	}

}
