/*
 * This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/**
 * 
 */
package bit.lsi.sar.nsg.swing;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.openscience.cdk.Atom;
import org.openscience.cdk.Molecule;
import org.openscience.cdk.applications.swing.MoleculeViewer2D;
import org.openscience.cdk.interfaces.IMolecule;
import org.openscience.cdk.renderer.Renderer2D;
import org.openscience.cdk.renderer.Renderer2DModel;







/**
 * @author lounkine
 *
 */
public class SmilesDepictionPanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = -2510259061054042572L;
	
	private static IMolecule dummymol = new Molecule();
	
	private IMolecule mol;
	private bit.lsi.sar.nsg.Molecule curmol;
	
	private Renderer2DModel rm2d;
	private MoleculeViewer2D mv2d = new MoleculeViewer2D();
	private JTextField smilesLabel = new JTextField(20);
	private JLabel nameLabel = new JLabel();
	private String curSmiles;
	
		
	public SmilesDepictionPanel (String inSmiles, String inMolname) {
		
		this();
		
	}
	
	public SmilesDepictionPanel (bit.lsi.sar.nsg.Molecule mol) {
		this();
		this.depictMol(mol);
				
	}
	
	public SmilesDepictionPanel () {
		
		super(new BorderLayout());
		
		//smilesLabel.setEnabled(false);
		
		
		
		smilesLabel.addKeyListener(new KeyListener() {

			@Override
			public void keyPressed(KeyEvent arg0) {
				// TODO Auto-generated method stub
				//smilesLabel.setText(curSmiles);
			}

			@Override
			public void keyReleased(KeyEvent arg0) {
				// TODO Auto-generated method stub
				//smilesLabel.setText(curSmiles);
			}

			@Override
			public void keyTyped(KeyEvent arg0) {
				// TODO Auto-generated method stub
				smilesLabel.setText(curSmiles);
			}
			
		});
		
		
		mv2d.setBackground(Color.white);
		mv2d.setPreferredSize(new Dimension(250,250));
		
		mv2d.getRenderer2DModel().setBackgroundDimension(mv2d.getPreferredSize());
		
		
		//sp.setInterrupted(true);
		
		this.setPreferredSize(new Dimension(250,300));
		
		this.add(nameLabel, BorderLayout.NORTH);
		this.add(mv2d, BorderLayout.CENTER);
		this.add(smilesLabel, BorderLayout.SOUTH);
		
		
					
	}
	
	
	
	
	
	private void doDepict (String inMolname) {
				
		rm2d = mv2d.getRenderer2DModel();
		
		
		rm2d.setBondDistance(100 / mol.getAtomCount());
		rm2d.setShowImplicitHydrogens(false);
		//rm2d.setShowEndCarbons(true);
		rm2d.setShowExplicitHydrogens(false);
		
		rm2d.setShowAromaticityInCDKStyle(true);
		rm2d.setShowAromaticity(true);
		rm2d.setBondWidth(1);
		rm2d.setUseAntiAliasing(true);
		rm2d.setDrawNumbers(false);
		
		
		
		mv2d.setAtomContainer(mol);
		
			
	}

	public Renderer2D getRenderer() {
		return mv2d.renderer;
	}
	
	public  MoleculeViewer2D getMoleculeViewer() {
		return mv2d;
	}
	
	public void clear() {
		
		mol = dummymol;
		mol.addAtom(new Atom("H"));
		doDepict("No Molecule selected");
		
	}
	
	
	public void depictMol(bit.lsi.sar.nsg.Molecule mol) {
		
		if (curmol == null || (mol != null && !curmol.equals(mol))) {
		
			curmol = mol;
			this.mol = mol.getCDKMol();
			
			curSmiles = mol.getSmiles();
			smilesLabel.setText(curSmiles);
			nameLabel.setText(mol.toString());
			
			
			doDepict(mol.toString());
		}
	}
	
	public bit.lsi.sar.nsg.Molecule getDepictedMol() {
		return curmol;
	}
	
	
}
